#this script is used to set up Linux Base environment

#get default size of core files in accordance to the current Hardening policy
core_limits="/etc/opt/nortel/base/corefile_limits"
if [ -r $core_limits ]; then
   source $core_limits
else
   #if the Hardening setup file does not exist, disable core files.
   CORESOFTLIMIT=0
   COREHARDLIMIT=0
fi

true=1
coresizeold=`ulimit -S -c`
apply_hard_rule=0

if [ "$CORESOFTLIMIT" == "0" ]; then
   #Core files creation is disabled by Hardening; zero soft and hard limits explicitly 
   apply_hard_rule=1
else
   if [ -r /opt/nortel/base/lib/common_functions.pm ]; then
      /usr/bin/perl -e 'use lib "/opt/nortel/base/lib"; use common_functions; exit is_debug_mode();'
      if [ $? -eq $true ]; then
         CORESOFTLIMIT="unlimited"
         COREHARDLIMIT="unlimited"
      fi
   fi

   if [ "$coresizeold" == "unlimited" ]; then
      #get limits from the Hardening settings
      apply_hard_rule=1
   elif [ "$coresizeold" == "0" ]; then
      #if core files size limit has not been sent by the parent process
      apply_hard_rule=1
   elif [ "$CORESOFTLIMIT" != "unlimited" ] && [ "$coresizeold" -gt "$CORESOFTLIMIT" ]; then
      #Hardening settings reduce core files size limits
      apply_hard_rule=1
   fi
fi

if [ "$apply_hard_rule" == "1" ]; then
   ulimit -S -c $CORESOFTLIMIT  > /dev/null 2>&1
   ulimit -H -c $COREHARDLIMIT  > /dev/null 2>&1
fi

unset apply_hard_rule
unset true
unset coresizeold
unset core_limits
unset CORESOFTLIMIT
unset COREHARDLIMIT

#set readonly timeout for all system accounts
TMOUT=900
readonly TMOUT
export TMOUT

#set 'non secure' mask only for nortel account
username=`/usr/bin/id -un`
if [ "$username" == "nortel" ]; then
    umask 007
else
    umask 027
fi

if [ -z "${PERLLIB}" ]; then
    PERLLIB=/opt/nortel/base/lib
else
    PERLLIB=/opt/nortel/base/lib:${PERLLIB}
fi
export PERLLIB

if [ -z "${LD_LIBRARY_PATH}" ]; then
    LD_LIBRARY_PATH=/opt/nortel/base/patchutil/binary/
else
    LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:/opt/nortel/base/patchutil/binary/
fi
export LD_LIBRARY_PATH

groups_list=`/usr/bin/id -Gn`

LESSOPEN=|/usr/bin/lesspipe.sh %s

# do not add Linux Base tools for root account
if [ "$username" != "root" ]; then
    # save the default paths
    default_paths=$PATH
    # set correct PATH variable
    PATH=/opt/nortel/common/bin/
    for grp in $groups_list ; do
        if [ "$grp" = "systemadmin" ]; then
            PATH=$PATH:/opt/nortel/groups/systemadmin/bin
        fi
        if [ "$grp" = "backupadmin" ]; then
            PATH=$PATH:/opt/nortel/groups/backupadmin/bin
        fi
        if [ "$grp" = "dbadmin" ]; then
            PATH=$PATH:/opt/nortel/groups/dbadmin/bin
        fi
        if [ "$grp" = "logadmin" ]; then
            PATH=$PATH:/opt/nortel/groups/logadmin/bin
        fi
        if [ "$grp" = "maintadmin" ]; then
            PATH=$PATH:/opt/nortel/groups/maintadmin/bin
        fi
        if [ "$grp" = "patchadmin" ]; then
            PATH=$PATH:/opt/nortel/groups/patchadmin/bin
        fi
        if [ "$grp" = "securityadmin" ]; then
            PATH=$PATH:/opt/nortel/groups/securityadmin/bin
        fi
        if [ "$grp" = "timeadmin" ]; then
            PATH=$PATH:/opt/nortel/groups/timeadmin/bin
        fi
        if [ "$username" != "nortel" ]; then
	   LESSOPEN=
	fi
    done
    unset grp
    # add default paths to the nortel user
    if [ "$username" == "nortel" -a -n "$default_paths" ]; then
        PATH=$PATH:$default_paths
    fi
fi
unset groups_list
export PATH
export LESSOPEN

