#!/usr/bin/perl
##########################################################
## @file cPcmd.pl
# @brief Invokes cPcmd($hostname, $ipaddr) function from base_pm module.
# @details
# @param hostname - target hostname to be patched
# @param IP - ip-address of the target host
# @author Sergey Urlin
# @date July 2008
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
# @note This script is run by Patch Manager as follows: cPcmd.pl -hostname <hostname> -IP <ip>

use lib "/opt/nortel/base/lib";
use strict;
use warnings;
use baselog;
use base_pm;
use common_variables;
use Getopt::Long;
use CommonInstallUtils;

sub cPatchCommand();
exit(cPatchCommand());

###############################################################################
## @function cPatchCommand($hostname, $ipaddr)
# @brief   Checks input parameters and executes cPcmd function from base_pm
# @details none.
# @param   $hostname - target hostname. Is used to resolve directory from
#          where to read patch.xml file.
# @param   $ipaddr - IP address associated with the host.
# @return  ERROR - if cPcmd function of the base_pm returns FALSE
# @return  OK - if cPcmd function of the base_pm returns TRUE
# @date    July 2008
sub cPatchCommand()
{
   my $hostname;
   my $ipaddr;
   GetOptions("hostname:s" => \$hostname,
              "IP:s"       => \$ipaddr);

   # check for validity of $hostname and $ipaddr
   if ((not defined $hostname) or ($hostname eq ''))
   {
      baselog($LOG_ERROR, 'Hostname is undefined.');
      return $ERROR;
   }
   if ((not defined $ipaddr) or ($ipaddr eq ''))
   {
      baselog($LOG_ERROR, 'IP address is undefined.');
      return $ERROR;
   }

   if (!validateIP($ipaddr))
   {
      baselog($LOG_ERROR, "Invalid IP address: $ipaddr.");
      return $ERROR;
   }
   my $patchObj = base_pm::new($hostname, $ipaddr);
   if (not defined $patchObj)
   {
      baselog($LOG_ERROR, "Can't create patch object.");
      return $ERROR;
   }

   if (not $patchObj->cPcmd())
   {
      return $ERROR;
   }
   undef $patchObj;
   return $OK;
} ## end sub cPatchCommand()


