#!/usr/bin/perl

## @file monitinit.pl
# @brief   Performs monit initialization operation
# @details Checks for enable flag for all applications and enable/disable monit for them
# @param   none
# @return $ERROR if error
# @return $OK    if success
# @author Alex Arkhipov
# @date October 20, 2008

use warnings;
use strict;

use lib '/opt/nortel/base/lib';

use common_variables;
use common_functions;

use install_common;
use baselog;
use base_apps_service;
use base_system;
use base_semctl;

# Maximum timeout that we keep semaphore
my $MAX_TIMEOUT = 60;

## @skipline
sub main();

exit(main());

sub main()
{
   my $ps_cmd = ['ps', '-C', 'monit'];
   baselog($LOG_INFO, 'Monitinit started');

   my $semmngr = base_semctl::new('Monitinit');
   if (!defined($semmngr))
   {
      baselog($LOG_ERROR, "Cannot create semaphore manager object ($!).");
      return $ERROR;
   }

   if (!$semmngr->sem_take($SEM_APPSTART, $WAIT_FLAG, $MONITSTARTUP_SEM_WAIT_TIME))
   {
      baselog($LOG_ERROR, 'Monitinit operation aborted.');
      undef $semmngr;
      return $ERROR;
   } ## end if (!$semmngr->sem_take...

   # Check if monit is running
   my $retval;
   my $counter = 0;
   while (++$counter < $MAX_TIMEOUT)
   {
      if (do_system($ps_cmd, {$dsRETVAL => \$retval, $dsNOERROR => $TRUE}))
      {
         if ($retval == $OK)
         {
            last;
         }
      } ## end if (do_system($ps_cmd,...
      else
      {
         baselog($LOG_ERROR, 'The do_system returns error. Do not check retval.');
      }
      baselog($LOG_INFO, 'Monit is not running. Sleeping...');
      sleep 1;
   } ## end while (++$counter < $MAX_TIMEOUT...

   my $app = new base_apps_service({$APPL_TYPE => $TYPE_ALL_APPL});
   if (not defined $app)
   {
      baselog($LOG_ERROR, 'Cannot create base_apps_service object. Aborted monitinit operation');
      undef $semmngr;
      return $ERROR;
   } ## end if (not defined $app)
   $app->monitinit();
   undef $app;

   if (!$semmngr->sem_give($SEM_APPSTART))
   {
      baselog($LOG_WARNING, "Cannot release appstart semaphore.");
   }
   undef $semmngr;

   baselog($LOG_INFO, 'Monitinit is finished successfully');
   return $OK;
} ## end sub main()

