#!/usr/bin/perl
####################################################################
#
## @file syslogFacilitySet.pl
# @brief This is implementation of syslogFacilitySet command.
# @details The command sets syslog facility for given application.
#          To take effect the application should be restarted.
# @param $_[0] - process
# @param $_[1] - facility
# @author Alexander Khoroshilov
# @date December 2007
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
#
####################################################################

=pod

=head1 NAME

B<syslogFacilitySet>

=head1 SYNOPSIS

B<syslogFacilitySet> [B<--help>] B<PROCESS> B<FACILITY>

=head1 DESCRIPTION

The command sets syslog facility for given process.
To take effect the process may need restart.

=head1 OPTIONS

=over 4

=item B<PROCESS>

Process for which changing facility. To obtain list of allowable processes start B<syslogShow> command without arguments.

=item B<FACILITY>

New facility.

Allowable facilities:

  LOCAL0
  LOCAL1
  LOCAL2
  LOCAL3
  LOCAL4
  LOCAL5
  LOCAL6
  LOCAL7

=item B<--help>

Print this help message and exit.

=back

=head1 EXAMPLES

$ syslogFacilitySet linuxbase LOCAL6

To take effect linuxbase may need restart.

=head1 SEE ALSO

B<syslogShow>, B<syslogLevelSet> commands.

=cut

use lib "/opt/nortel/base/lib";
use strict;
use warnings;
use common_variables;
use syslog_config;
use baselog;
use Pod::Usage;

sub main();

exit(main());

## @fn main()
# @brief Entry point.
# @details
# @param None
# @return $OK - on success
# @return $ERROR - on error
# @author Alexander Khoroshilov
# @date December 2007
sub main()
{
   set_subsystem("Base");
   if ((defined(@ARGV)) && (($ARGV[0] eq "-h") || ($ARGV[0] eq "--help")))
   {
      pod2usage(
                -exitval => 0,
                -verbose => 1,
                -output  => \*STDOUT
               );
   } ## end if ((defined(@ARGV)) &&...
   else
   {
      my $file = new syslog_config($SYSLOG_CFG_FILE);
      unless (defined($file))
      {
         baselog($LOG_ERROR, "Cannot create syslog_config object for $SYSLOG_CFG_FILE\n", $TRUE);
         return $ERROR;
      }
      unless ($file->set_facility(@ARGV))
      {
         return $ERROR;
      }
   } ## end else [ if ((defined(@ARGV)) &&...
   return $OK;
} ## end sub main()
