#!/usr/bin/perl
####################################################################
#
## @file syslogLevelSet.pl
# @brief This is implementation of syslogLevelSet command.
# @details The command sets syslog level for given application
#          and task to suppress log messages with less important levels.
# @param $_[0] - process
# @param $_[1] - task
# @param $_[2] - level
# @author Alexander Khoroshilov
# @date December 2007
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
#
####################################################################

=pod

=head1 NAME

B<syslogLevelSet>

=head1 SYNOPSIS

B<syslogLevelSet> [B<--help>] B<PROCESS> B<TASK> B<LEVEL>

=head1 DESCRIPTION

The command sets syslog level for given process and task to
suppress log messages with less important levels.

=head1 OPTIONS

=over 4

=item B<PROCESS>, B<TASK>

Process and task for which changing level. To obtain list of allowable processes with their tasks start B<syslogShow> command without arguments.

=item B<LEVEL>

New level.

Allowable levels:

  NONE
  EMERG
  ALERT
  CRIT
  ERROR
  WARNING
  NOTICE
  INFO
  DEBUG

=item B<--help>

Print this help message and exit.

=back

=head1 EXAMPLES

$ syslogLevelSet linuxbase Base_Patch DEBUG

=head1 SEE ALSO

B<syslogShow>, B<syslogFacilitySet> commands.

=cut

use lib "/opt/nortel/base/lib";
use strict;
use warnings;
use common_variables;
use syslog_config;
use baselog;
use Pod::Usage;

sub main();

exit(main());

## @fn main()
# @brief Entry point.
# @details
# @param None
# @return $OK - on success
# @return $ERROR - on error
# @author Alexander Khoroshilov
# @date December 2007
sub main()
{
   set_subsystem("Base");
   if ((defined(@ARGV)) && (($ARGV[0] eq "-h") || ($ARGV[0] eq "--help")))
   {
      pod2usage(
                -exitval => 0,
                -verbose => 1,
                -output  => \*STDOUT
               );
   } ## end if ((defined(@ARGV)) &&...
   else
   {
      my $file = new syslog_config($SYSLOG_CFG_FILE);
      unless (defined($file))
      {
         baselog($LOG_ERROR, "Cannot create syslog_config object for $SYSLOG_CFG_FILE\n", $TRUE);
         return $ERROR;
      }
      unless ($file->set_level(@ARGV))
      {
         return $ERROR;
      }
   } ## end else [ if ((defined(@ARGV)) &&...
   return $OK;
} ## end sub main()
