#!/usr/bin/perl
####################################################################
#
## @file syslogShow.pl
# @brief This is implementation of sysLogShow command.
# @details The command shows list of applications with
#          associated facilities and permitted syslog levels
#          for tasks per application.
# @param $_[0]... - process to show (optional)
# @author Alexander Khoroshilov
# @date December 2007
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.
#
####################################################################

=pod

=head1 NAME

B<syslogShow>

=head1 SYNOPSIS

B<syslogShow> [B<--help>] [B<PROCESS>]... 

=head1 DESCRIPTION

The command shows list of processes with syslog facilities and
syslog levels of tasks per process.

=head1 OPTIONS

=over 4

=item B<PROCESS>...

List of processes to show. If none given, show all processes.

=item B<--help>

Print this help message and exit.

=back

=head1 EXAMPLES

$ syslogShow linuxbase

   Process: linuxbase
   Facility: LOCAL7
   ----------------------------
   Task                 Level

   Base                 INFO
   Base_Appinstall      INFO
   Base_Appstartup      INFO
   Base_Backup          INFO
   Base_Patch           INFO

=head1 SEE ALSO

B<syslogFacilitySet>, B<syslogLevelSet> commands.

=cut

use lib "/opt/nortel/base/lib";
use strict;
use warnings;
use common_variables;
use syslog_config;
use baselog;
use Pod::Usage;

sub main();

exit(main());

## @fn main()
# @brief Entry point.
# @details
# @param None
# @return $OK - on success
# @return $ERROR - on error
# @author Alexander Khoroshilov
# @date December 2007
sub main()
{
   set_subsystem("Base");
   if ((defined(@ARGV)) && (($ARGV[0] eq "-h") || ($ARGV[0] eq "--help")))
   {
      pod2usage(
                -exitval => 0,
                -verbose => 1,
                -output  => \*STDOUT
               );
   } ## end if ((defined(@ARGV)) &&...
   else
   {
      my $file = new syslog_config($SYSLOG_CFG_FILE);
      unless (defined($file))
      {
         baselog($LOG_ERROR, "Cannot create syslog_config object for $SYSLOG_CFG_FILE\n", $TRUE);
         return $ERROR;
      }
      unless ($file->show(@ARGV))
      {
         return $ERROR;
      }
   } ## end else [ if ((defined(@ARGV)) &&...
   return $OK;
} ## end sub main()
