## @file base_led.pm
# @brief This module is Linux Base library for LED operations.
# @details
# @author Ekaterina Bazhanova
# @date December 2008
# @version 1.00 - Initial version
package base_led;

use lib "/opt/nortel/base/lib";

use common_variables;
use common_functions;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&turn_led_green);

use strict;
use warnings;
use baselog;
use base_system;

require 'sys/ioctl.ph';

our $VERSION = '1.00';

our $CPPMLEDFLAG = '/etc/opt/nortel/cppmUtil/.ledflag';
our $CPPMLEDDEV  = '/dev/cppmLed';

## @skipline
sub turn_led_green();

###############################################################################
## @fn       turn_led_green()
# @brief     Turn status LED green for CPPM
# @details   None.
# @return    $TRUE    - successfully condition
# @return    $FALSE   - indicates an error condition.
# @author    Ekaterina Bazhanova
# @date      December 2008
sub turn_led_green()
{
   my $ledfh   = undef;
   my $ledflag = undef;

   if (!-f $CPPMLEDFLAG)
   {  ## if ledflag file does not exist
      unless (open($ledfh, '<', $CPPMLEDDEV))
      {
         baselog($LOG_ERROR, "Cannot open $CPPMLEDDEV");
         return $FALSE;
      }
      unless (ioctl ($ledfh, 0x07, 1))
      {
         baselog($LOG_ERROR, "Cannot turn status LED green");
         unless (close($ledfh))
         {
            baselog($LOG_ERROR, "Cannot close $CPPMLEDDEV");
         } 
         return $FALSE;
      }
      unless (close($ledfh))
      {
         baselog($LOG_ERROR, "Cannot close $CPPMLEDDEV");
         return $FALSE;
      } 
      unless (do_system([$TOUCH, $CPPMLEDFLAG]))
      {
         baselog($LOG_ERROR, "Cannot create $CPPMLEDFLAG");
         return $FALSE;
      }

   } ## end if (!-f $CPPMLEDFLAG) ...
   return $TRUE; 
}


1;

__END__

=head1 NAME
base_led - Linux Base library for LED operations.

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 METHODS

=over 4

=item new


=over 4

=item *

=head1 AUTHOR

Ekaterina Bazhanova

=back

