############################################################
## @file common_variables.pm
# @brief Common data variables for all Linux Base tools
# @note Please put any new common variables here
# @note Please follow existing style when add something
# @version 1.00 - Initial version
# @version 1.01 - The following variables were added:
#   @li SFTP_LOGIN
#   @li CONTROLJAILUTIL
#   @li SFTPJAILDIR
#   @li SFTPSERVER
#   @li CHROOT
#   @li JAILTEMPLATEDIR
#   @li CREATEJAILTEMPLATEUTIL
# @version 1.02 - The $ILOG var was removed as not used anymore
# @version 1.03 - Recovery Console Variables were added
# @version 1.04 - $RECOVER_HOMES_SCRIPT and $BACKUP_HOMES_SCRIPT have been removed
# @version 1.05 - $NO_BACKUP flag added
# @version 1.06 - Added %GROUPS, $NO_LOG
# @version 1.07 - Added $RMT_SYSRESTORE_CMD, $RMT_SYSBACKUP_CMD
# @version 1.08 - Added $BASENAME $DIRNAME
# @version 1.09 - Added $IPTABLES_RESTORE, $USERCFG
# @version 1.10 - Added $ABORT and $FAULT return values
# @version 1.11 - $SHUTDOWN command is added
# @version 1.12 - $REPACKAGEDIR added
# @version 1.13 - Added $NORTEL_LINUX_BASE_APP_NAME
# @version 1.14 - Added $IPTABLES_SERVICE
# @version 1.15 - $CONFVER_CONTENT $RPMDB_VERIFY $RPMDATABASEDIR %CONFVER_STATES $CONFVER_INST was added.
# @version 1.16 - $PER_SCRIPT_KEY $PER_APPL_KEY added
# @version 1.17 - Added $SSH_ALIVE_INTERVAL $SSH_TCP_KEEP_ALIVE $SSH_CONNECT_TO
# @version 1.18 - $MAX_YEAR $MIN_YEAR added
# @version 1.19 - Added $ELAN_GW, $TLAN_GW
# @version 1.20 - Added $SIGNALS_IGNORE_COUNTER
# @version 1.21 - $STAT added
# @version 1.22 - Added $LOCALHOST
# @version 1.23 - $SSH_ALIVE_COUNT_MAX added
# @version 1.24 - Added @JAIL_DIRS, $PASSWDFILE, $GROUPFILE, $SKEL_DIR. $CONTROLJAILUTIL has been removed
# @version 1.25 - Added $TEMP_RESTORE_DIR
# @version 1.26 - value of $MAN_CENTER has been changed
# @version 1.27 - Added $NTPDATE command
# @version 1.28 - Added $GROUPADD
# @version 1.29 - Changed %GROUPS to explicitly use 'nortel' rather than get the
#                 'nortel' group name via "getgrgid", since %GROUPS is used for
#                 post-install stage also, at which time the 'nortel' group does
#                 not exist in /etc/group
# @version 1.30 - Added $FIRST_LOGIN_GUIDE
# @version 1.31 - $CONFIGURE_XML_SCHEME_PATH and %XML_PATHS were added
# @version 1.32 - Added $IPTABLES_SAVE
# @version 1.33 - Added $CLEANUP_PATCHDIR
# @version 1.34 - Added $LCHAGE, $PASS_UNLIM_VALUE, Added $NOAUTO
# @version 1.35 - Added $PI_CONTROL_RPM definition.
# @version 1.36 - Added $MIN_MEMORY_SIZE and $MIN_HARD_DRIVE_SIZE constants
# @version 1.37 - Change $MIN_HARD_DRIVE_SIZE to 38000
# @version 1.38 - Added $OS_RPM and $DEPLOYMENT_RPM definitions.
# @version 1.39 - Added $FLASHROM and $BASE_DATA_DIR constants
# @version 1.40 - Added $ETHTOOL
# @version 1.41 - Added $ENHANCED_HARDENING_REAPPLY_POST_PATCHING
# @version 1.42 - Added $CREATE_USB_MOUNTPOINT
# @version 1.43 - Added $PATCH_CHOWN $PATCH_CHMOD
# @version 1.44 - Added $HOSTS_TABLE_FILE and $UPDATE_HOSTS.
# @version 1.45 - Added $TTY1, $YES, $NO
# @version 1.46 - Added $REPLACE_NTP_CONFIGS
# @version 1.47 - Added a number of variables in support of the SSH/HD target upgrade

package common_variables;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw($TRUE $FALSE $OK $ERROR $DONE $FAULT $ABORT $APP_CFG_ROOT $APPLCFG $BASE_CONFIG_DIR $NORTEL_CRON_CONF
  $LOGDIR $BACKUPDIR $ADMIN_PARTITION $SAVESTDIN $SYSLOG_CFG_FILE $FUNCTIONS_LIB $REL_NAME
  $BASE_REL_NAME $INSTALLEDCONFIG $CONFVER $BASEAPPCONFVER $INSTCFG $BASEAPPINSTCFG $BASEAPPDIR
  $APPINSTALL $BASEFIRSTBOOTUP $MOTDFILE $RMT_DEPLOY_CMD $BASESYSINFO $DEPLOYMENTCFG $BASEOSPROP
  $ARCHDIR $ARCHVER $RPMBACKUP $UPGRADE_BACKUP $UPGRADE_RESTORE $LBINFO $CLEAR_BACKUP $APP_SERVICE
  $APP_DEPLOY $DHCPD_CONFIG $IF_CTRL $IF_UP $IF_DOWN $BASIC_HARDENIG_REAPPLY_POST_PATCHING
  $SHOW_BACKUP_INFO $SRV_START $SRV_STOP $SRV_STATUS $SRV_RESTART $SRV_RELOAD $SRV_ON $SRV_OFF
  $BASE_APPS $DEBUG_MODE $APP_INSTALLED $APP_VERSION $APP_PARTIAL $MAN_CENTER $MAN_RELEASE
  $SENDSNMPTRAP $ALARM $APPSTARTUP $ARPING $AUDITCTL $AWK $BASH $CAT $CHATTR $CHMOD $CHOWN $CLEAR $CP
  $CRONTAB $CHRT $CHKCONFIG $DATE $DF $ECHO $EJECT $FIND $FINDBYPROP $GENCRONTAB $GETPROPERTY $GREP
  $GZIP $HEAD $HOSTNAME $HWCLOCK $IFCONFIG $IPTABLES $IPTABLES_RESTORE $KILLALL $LN $LS
  $LSATTR $MAN $MD5SUM $MKDIR $MKNOD $MOUNT $MV $NETSTAT $NEWALIASES $NICE $NM $NROFF $PASSWD
  $PARTED $PERL $PING $PING6 $POD2MAN $PS $PWD $READLINK $REBOOT $RLOGIN $RM $ROUTE $RPM $RPMQUERY
  $RUNUSER $SCP $SED $SERVICE $SSH $STTY $SH $SHUTDOWN $SU $SUDO $SUDOEDIT $SUDO_NORTEL $SYNC $SYSBACKUP
  $SYSCTL $TAIL $TAR $TOUCH $UMOUNT $UNAME $UNZIP $WALL $USERDEL $USERHELPER $USERMOD
  $WC $WHOAMI $ZIP $NORTEL_UID $NORTEL_GID $ROOT_UID $NORTEL_ACCT $DEPLOY_ACCT $ROOT_ACCT
  $DEPLOY_ACCT $ROOT_ACCT $ROOT_GID $SYSRESTORE $FILE %XML_PATHS $CONFIGURE_XML_SCHEME_PATH
  $SYSBACKUPUTIL $FSTAB $PARTITIONS $RMT_UPGRADE_CMD $RMT_UNDEPLOY_CMD $KCV $ACCOUNTS_CFG
  $SMCRYPT $USERADD $RBASH $DMESG $BASERECOVERY_FLAG $TEST $ENHANCED_HARDENING_REAPPLY_POST_PATCHING
  $FIREWALL_RULES_FILE $MORE $NORTEL_GRP $ROOT_GRP $FREE $MASTER_CFG_ROOT $MASTERCFG
  $RECDIR $ADMINDIR $BOOTDIR $GRUBDIR $ROOTDIR $VARDIR $GRUBCONF $SHADOWFILE $USERINFO $TTY
  $DD $E2LABEL $FDISK $FSCK $GRUB $GRUBBY $GRUBINST $NANO $NO_BACKUP $USERCFG $PINS_SEM_WAITTIME
  $SFTP_LOGIN $SFTPSERVER $CHROOT $JAILTEMPLATEDIR @JAIL_DIRS $CREATE_JAIL_TEMPLATE_UTIL
  %GROUPS $NO_LOG $REPACKAGEDIR $RMT_SYSRESTORE_CMD $RMT_SYSBACKUP_CMD $CONFVER_CONTENT $RPMDB_VERIFY
  $DIRNAME $BASENAME $NORTEL_LINUX_BASE_APP_NAME $IPTABLES_SERVICE $RPMDATABASEDIR %CONFVER_STATES $PER_SCRIPT_KEY $PER_APPL_KEY
  $SSH_ALIVE_INTERVAL $SSH_TCP_KEEP_ALIVE $SSH_ALIVE_COUNT_MAX $SSH_CONNECT_TO $MAX_YEAR $MIN_YEAR $ELAN_GW $TLAN_GW $STAT
  $SIGNALS_IGNORE_COUNTER $LOCALHOST $PASSWDFILE $GROUPFILE $SKEL_DIR $TEMP_RESTORE_DIR
  $TIME_SETUP $TIME_RECIPIENT $NOT_TIME_RECIPIENT $NTPDATE $GROUPADD $FIRST_LOGIN_GUIDE $CLEANUP_PATCHDIR $UPDATE_HOSTS
  $FAILLOG $IPTABLES_SAVE $NOAUTO $PASS_UNLIM_VALUE $LCHAGE $PI_CONTROL_RPM $MIN_MEMORY_SIZE $MIN_HARD_DRIVE_SIZE $HOSTS_TABLE_FILE
  $OS_RPM $DEPLOYMENT_RPM $FLASHROM $BASE_DATA_DIR $CREATE_USB_MOUNTPOINT $REPLACE_NTP_CONFIGS
  $ETHTOOL $PATCH_CHOWN $PATCH_CHMOD $PATCHINGDIR $PATCH_PLOAD $TTY1 $YES $NO
  $RPCINFO $RMT_ARCH_PATH $RMT_SDS_PATH $DMS_ARCH_PATH $DMS_SDS_PATH
  $BOOT_K $ADMIN_K $ROOT_K $HOME_K $OPT_K $TMP_K $VAR_K $LOG_K $SWAP_K $COREDUMP_K $FSPART_TXT $DU
  );

our $VERSION = '1.47';

#### VARIABLES

our ($TRUE,    $FALSE)   = (1, 0);
our ($ELAN_GW, $TLAN_GW) = (0, 1);
our ($OK, $ERROR, $DONE, $FAULT, $ABORT) = (0, 1, 2, 3, 4);
our $APP_CFG_ROOT     = "/etc/opt/nortel";            #application config path
our $APPLCFG          = "configure.xml";              #application config file
our $BASE_CONFIG_DIR  = "/etc/opt/nortel/base";
our $NORTEL_CRON_CONF = "/home/nortel/.nortel.cron";  #nortels cron config. file
our $LOGDIR           = "/var/log/nortel/base";
our $BACKUPDIR        = "/admin/nortel/backup";
our $ADMIN_PARTITION  = "/admin";
our $SAVESTDIN        = undef;                        # global variable to store STDIN
our ($TIME_RECIPIENT, $NOT_TIME_RECIPIENT) = (5, 6);
our $NOAUTO = "NoAuto";                               #used by Linux Patching for the
                                                      #removable flag in the patch header.

our $YES = "yes";
our $NO  = "no";

# linuxbase data directory
our $BASE_DATA_DIR = "/opt/nortel/base/data";

# rpm used to validate pi-control fru patches.
our $PI_CONTROL_RPM = "nortel-cs1000-pi-control";

# rpm used to allow deployment of patch not associated with any RPM.
our $OS_RPM = "nortel-cs1000-OS";

# rpm used to allow deployment of patch not associated with any RPM,
# but are specifically for use by the Deployment Manager
our $DEPLOYMENT_RPM = "nortel-cs1000-deployment";

#CS1000 syslog config file
our $SYSLOG_CFG_FILE = "/etc/opt/nortel/base/log/cs1000_syslog.ini";

#Common cron scheduler file
our $GENCRONTAB = "/etc/crontab";

#system year range
our $MAX_YEAR = 2037;
our $MIN_YEAR = 2004;

# nortel-linuxbase application name
our $NORTEL_LINUX_BASE_APP_NAME = 'base';

#iptables service name
our $IPTABLES_SERVICE = "iptables";

#Library contains functions common for process of starting/stopping services
our $FUNCTIONS_LIB = "/etc/rc.d/init.d/functions";

#Release name starts with
our $REL_NAME = "nortel-cs1000-";

#Linux Base release name for version checking
our $BASE_REL_NAME = $REL_NAME . "linuxbase";

#Recover Base configuration and Base Applications data flag
our $BASERECOVERY_FLAG = "/admin/.recoverBase";

#SDS Recover Base configuration and Base Applications data flag
our $SDSCLIENTINSTALL_FLAG = "/admin/.sdsClient";

# Deployment Manager Server remote deployment folders
our $RMT_ARCH_PATH = "/var/tmp/tmp";  ## Remote temporary Archive Folder
our $RMT_SDS_PATH  = "/var/log";      ## Remote temporary DM Repository
our $DMS_ARCH_PATH = "/var/tmp";      ## DM Server temporary Archive Folder
our $DMS_SDS_PATH  = "/var/sds";      ## DM Server DM Repository

# SSH/HD Target Upgrade variables:
# - Specific mount pt names/labels for which we need to determine partition
#   numbers for. These will represent our hash keys to use.
#
our $BOOT_K     = '/boot';
our $ADMIN_K    = '/admin';
our $ROOT_K     = '/';
our $HOME_K     = '/home';
our $OPT_K      = '/opt';
our $TMP_K      = '/tmp';
our $VAR_K      = '/var';
our $LOG_K      = '/var/log';
our $SWAP_K     = 'swap';
our $COREDUMP_K = '/var/coredump';

# - rmtSdcUpgrade() generated partition information file to be used by
#   "SetupPartitions.pm::buildPartInfoStrings()"
our $FSPART_TXT = "fspart.txt";


our $INSTALLEDCONFIG = 'installedconfig';

#path to file which stores installed supportedConfig name
our $CONFVER = "/admin/nortel/install/$INSTALLEDCONFIG";

our $CONFVER_CONTENT = {APP_INSTALLED => 'undef', APP_VERSION => 'undef'};

our %CONFVER_STATES = (
                       COMPLETE      => 'COMPLETE',
                       STARTED       => 'STARTED',
                       RPM_INSTALLED => 'RPM_INSTALLED',
                       UNINSTALLING  => 'UNINSTALLING',
                      );

#path to file which stores installed supportedConfig name
our $BASEAPPCONFVER = "/admin/nortel/install/installedbaseappconfig";

#path to the xml scheme file for configure.xml
$CONFIGURE_XML_SCHEME_PATH = "/etc/opt/nortel/base/xmlschemes/configure.scheme.xml";

#LinuxBase firewall rules
our $FIREWALL_RULES_FILE = '/var/opt/nortel/base/appconfig/base_rules.xml';

#installation configuration file
our $INSTCFG = "install.xml";

#Base Applications configuration file
our $BASEAPPINSTCFG = "baseapps-install.xml";

#Base Applications RPMs path
our $BASEAPPDIR = "/var/opt/nortel/base-apps/";

#Back-up location for system RPMS
our $REPACKAGEDIR = "/var/spool/repackage";

#RPM system database
our $RPMDATABASEDIR = "/var/lib/rpm/";

# Path to the "appinstall" util
our $APPINSTALL = "/opt/nortel/base/appinstall/bin/appinstall";

# Path to the "sysrestore" util
our $SYSRESTORE = "/opt/nortel/base/recovery/bin/sysrestore";

# Path to the "sysbackup" util
our $SYSBACKUPUTIL = "/opt/nortel/base/recovery/bin/sysbackup";

#First bootup lock-file
our $BASEFIRSTBOOTUP = "/etc/opt/nortel/base/base_first_bootup_flag_file";

# sftp-jail template directory
our $JAILTEMPLATEDIR = "/var/opt/nortel/base/jail_files";

# Directories required for work of SFTP-jail
our @JAIL_DIRS = ('dev', 'etc/pam.d', 'etc/security', 'bin', 'sbin', 'lib', 'usr');

our $MOTDFILE   = "/etc/motd";
our $FSTAB      = "/etc/fstab";
our $PARTITIONS = "/proc/partitions";

# deploy commands
our $RMT_DEPLOY_CMD     = '/opt/nortel/base/bin/rmtHostDeploy';
our $RMT_UPGRADE_CMD    = '/opt/nortel/base/bin/rmtHostUpgrade';
our $RMT_UNDEPLOY_CMD   = '/opt/nortel/base/bin/rmtHostUndeploy';
our $RMT_SYSRESTORE_CMD = '/opt/nortel/base/bin/rmtSysrestore';
our $RMT_SYSBACKUP_CMD  = '/opt/nortel/base/bin/rmtSysbackup';
our $BASESYSINFO        = '/opt/nortel/base/bin/baseSysinfo';
our $KCV                = '/opt/nortel/kcv/bin/kcv';

#deployment configuration file
our $DEPLOYMENTCFG = 'deployment.xml';

# baseOs.properties file for ECM and LinuxBase Element registration
our $BASEOSPROP = '/opt/nortel/Jboss-Quantum/server/default/conf/baseOs.properties';

#path to archived install.xml
our $ARCHDIR = "/admin/nortel/install";
our $ARCHVER = $ARCHDIR . "/" . $INSTCFG;

# file containing information about added users
our $ACCOUNTS_CFG = "accounts.xml";

#applications rpm backup path
our $RPMBACKUP = "/var/opt/nortel/base/installed_rpms/" . ($ENV{'SURPMBACKUP'} ? 'SU' : '');

#upgrade local backup file
our $UPGRADE_BACKUP  = "/admin/nortel/upgrade_backup.tar.gz";
our $UPGRADE_RESTORE = "/admin/nortel/upgrade_restore.tar.gz";

#temporary directory to keep system backup/restore archives
our $TEMP_RESTORE_DIR = "/tmp/sys_restore";

#file nortel-linuxbase-info on run-time stage
our $LBINFO = "/admin/nortel-linuxbase-info";

#sudo_nortel keys
our $CLEAR_BACKUP          = "clear_backup";
our $APP_SERVICE           = "app_service";
our $APP_DEPLOY            = "app_deploy";
our $DHCPD_CONFIG          = "dhcpd_config";
our $IF_CTRL               = "if_ctrl";
our $SFTP_LOGIN            = "sftp_login";
our $TIME_SETUP            = "time_setup";
our $CLEANUP_PATCHDIR      = "cleanup_patchdir";
our $CREATE_USB_MOUNTPOINT = "create_usb_mountpoint";
our $PATCH_CHOWN           = "patching_chown";
our $PATCH_CHMOD           = "patching_chmod";
our $UPDATE_HOSTS          = "update_hosts";
our $REPLACE_NTP_CONFIGS   = "replace_ntp_configs";

#sudo_nortel arguments for IF_CTRL command
our $IF_UP                                    = "if_up";
our $IF_DOWN                                  = "if_down";
our $BASIC_HARDENIG_REAPPLY_POST_PATCHING     = "reapply_basic_hardening_post_patching";
our $ENHANCED_HARDENING_REAPPLY_POST_PATCHING = "reapply_enhanced_hardening_post_patching";

#baseSysinfo keys
our $SHOW_BACKUP_INFO = 'backup';

#Application startup service (not CLI command)
our $APPSTARTUP = "appstartup";

#arguments for startup service
our $SRV_START   = "start";
our $SRV_STOP    = "stop";
our $SRV_STATUS  = "status";
our $SRV_RESTART = "restart";
our $SRV_RELOAD  = "reload";

#arguments for services configuration
our $SRV_ON  = "on";
our $SRV_OFF = "off";

#appstart key
our $BASE_APPS = "baseapps";

#User-defined configuration name
our $USERCFG = "usercfg";

#signals ignore counter, used by signals_ignore(), signals_default()
our $SIGNALS_IGNORE_COUNTER = 0;

#patching constants
our $PINS_SEM_WAITTIME = 15;

#indication of debug mode
our $DEBUG_MODE = "DEBUG_MODE=1";

#Man pages configuration
our $MAN_CENTER  = "Nortel Contributed Documentation";
our $MAN_RELEASE = "CS1000 Linux Base";

#send SNMP trap tool
our $SENDSNMPTRAP = "/opt/nortel/Snmp-Daemon-TrapLib/bin/sendSnmpTrap";

#userinfo.txt file path
our $USERINFO = "/admin/userinfo.txt";

#shadow file path
our $SHADOWFILE = "/etc/shadow";

#passwd file
our $PASSWDFILE = '/etc/passwd';

#group file
our $GROUPFILE = '/etc/group';

# skel directory
our $SKEL_DIR = '/etc/skel';

# /etc/hosts file
our $HOSTS_TABLE_FILE = '/etc/hosts';

#grub.conf file path
our $GRUBCONF = "/boot/grub/grub.conf";

#Recovery Console
#Recovery Console Directory
our $RECDIR = "/tmp/recovery";

#admin directory
our $ADMINDIR = $RECDIR . "/admin";

#boot directory
our $BOOTDIR = $RECDIR . "/boot";

#grub directory
our $GRUBDIR = $BOOTDIR . "/grub";

#root directory
our $ROOTDIR = $RECDIR . "/root";

#var directory
our $VARDIR = $RECDIR . "/var";

#local host ip
our $LOCALHOST = "127.0.0.1";

#
# SSH & SCP Option Parameters
#
# ServerAliveInterval:
#   Sets a timeout interval in seconds after which if no data has been received
#   from the server, ssh will send a message through the encrypted
#   channel to request a response from the server.
our $SSH_ALIVE_INTERVAL = 1;

# TCPKeepAlive
#   Specifies whether the system should send TCP keepalive messages to the other
#   side.  If they are sent, death of the connection or crash of one of the
#   machines will be properly noticed.
our $SSH_TCP_KEEP_ALIVE = "yes";

# ConnectTimeout:
#   Specifies the timeout (in seconds) used when connecting to the ssh
#   server, instead of using the default system TCP timeout.  This value is used
#   only when the target is down or really unreachable, not when it refuses the
#   connection.
our $SSH_CONNECT_TO = 3;

# ServerAliveCountMax
#   Sets the number of server alive messages which may be sent without ssh receiving
#   any messages back from the server. If this threshold is reached while server alive
#   messages are being sent, ssh will disconnect from the server, terminating the session.
$SSH_ALIVE_COUNT_MAX = 30;

## @var $NO_BACKUP
# @breif backup or not for write_function()
our $NO_BACKUP = $TRUE;

## @var $NO_LOG
# @breif log or not INFO message, describing successfull changes
#        for change_permissions_and_ownership function
our $NO_LOG = $TRUE;

## @var $PER_SCRIPT_KEY
# @breif The default application restart behavior
# @detail The 'restart' action means to run all applications scripts in the
#   direct order with the 'restart' option.
our $PER_SCRIPT_KEY = 'per_script';

## @var $PER_APPL_KEY
# @breif Change the application restart behavior
# @detail The 'restart' action means 'stop' and 'start' action.
our $PER_APPL_KEY = 'per_appl';

## @var $FIRST_LOGIN_GUIDE
# @breif Text banner, which is displayed after first nortel login
# @detail Text banner, which is added ot motd file.
our $FIRST_LOGIN_GUIDE = "   To access the UCM Web page, go to:
       UCM URL: http://<FQDN|IP>";

#Master Firewall Control
our $MASTER_CFG_ROOT = "/var/opt/nortel/base/appconfig";
our $MASTERCFG       = "master_firewall_control.xml";

#this value is used by lchage command
our $PASS_UNLIM_VALUE = 99999;

# Minimum Hardware Requirements
our $MIN_MEMORY_SIZE     = 2 * 1024;
our $MIN_HARD_DRIVE_SIZE = 38000;

# Terminal Names
our $TTY1 = "tty1";

#### Common used tools in alphabetical order

our $ALARM                     = "/opt/nortel/base/bin/alarm";
our $ARPING                    = "/sbin/arping";
our $AUDITCTL                  = "/sbin/auditctl";
our $AWK                       = "/bin/awk";
our $BASENAME                  = "/bin/basename";
our $BASH                      = "/bin/bash";
our $CAT                       = "/bin/cat";
our $CHATTR                    = "/usr/bin/chattr";
our $CHMOD                     = "/bin/chmod";
our $CHOWN                     = "/bin/chown";
our $CHROOT                    = "/usr/sbin/chroot";
our $CLEAR                     = "/usr/bin/clear";
our $CP                        = "/bin/cp";
our $CREATE_JAIL_TEMPLATE_UTIL = "/opt/nortel/base/bin/create_jail_template.sh";
our $CRONTAB                   = "/usr/bin/crontab";
our $CHRT                      = "/usr/bin/chrt";
our $CHKCONFIG                 = "/sbin/chkconfig";
our $DATE                      = "/bin/date";
our $DD                        = "/bin/dd";
our $DF                        = "/bin/df";
our $DIRNAME                   = "/usr/bin/dirname";
our $DMESG                     = "/bin/dmesg";
our $DU                        = "/usr/bin/du";
our $E2LABEL                   = "/sbin/e2label";
our $ECHO                      = "/bin/echo";
our $ETHTOOL                   = "/sbin/ethtool";
our $FDISK                     = "/sbin/fdisk";
our $EJECT                     = "/usr/bin/eject";
our $FAILLOG                   = "/usr/bin/faillog";
our $FILE                      = "/usr/bin/file";
our $FIND                      = "/usr/bin/find";
our $FINDBYPROP                = "/usr/bin/hal-find-by-property";                # HAL util to find udi by property
our $FLASHROM                  = "/usr/sbin/flashrom";
our $FREE                      = "/usr/bin/free";
our $FSCK                      = "/sbin/fsck";
our $GETPROPERTY               = '/usr/bin/hal-get-property';                    # HAL util to get property by udi
our $GREP                      = "/bin/grep";
our $GROUPADD                  = "/usr/sbin/groupadd";
our $GRUB                      = "/sbin/grub";
our $GRUBBY                    = "/sbin/grubby";
our $GRUBINST                  = "/sbin/grub-install";
our $GZIP                      = "/bin/gzip";
our $HEAD                      = "/usr/bin/head";
our $HOSTNAME                  = "/bin/hostname";
our $HWCLOCK                   = "/sbin/hwclock";
our $IFCONFIG                  = "/sbin/ifconfig";
our $IPTABLES                  = "/sbin/iptables";
our $IPTABLES_RESTORE          = "/sbin/iptables-restore";
our $IPTABLES_SAVE             = "/sbin/iptables-save";
our $KILLALL                   = "/usr/bin/killall";
our $LCHAGE                    = "/usr/sbin/lchage";
our $LN                        = "/bin/ln";
our $LS                        = "/bin/ls";
our $LSATTR                    = "/usr/bin/lsattr";
our $MAN                       = "/usr/bin/man";
our $MD5SUM                    = "/usr/bin/md5sum";
our $MKDIR                     = "/bin/mkdir";
our $MKNOD                     = "/bin/mknod";
our $MORE                      = "/bin/more";
our $MOUNT                     = "/bin/mount";
our $MV                        = "/bin/mv";
our $NANO                      = "/usr/bin/nano";
our $NETSTAT                   = "/bin/netstat";
our $NEWALIASES                = "/usr/bin/newaliases";
our $NICE                      = "/bin/nice";
our $NM                        = "/usr/bin/nm";
our $NROFF                     = "/usr/bin/nroff";
our $NTPDATE                   = "/usr/sbin/ntpdate";
our $PASSWD                    = "/usr/bin/passwd";
our $PARTED                    = "/sbin/parted";
our $PERL                      = "/usr/bin/perl";
our $PING                      = "/bin/ping";
our $PING6                     = "/bin/ping6";
our $POD2MAN                   = "/usr/bin/pod2man";
our $PS                        = "/bin/ps";
our $PWD                       = "/bin/pwd";
our $RBASH                     = "/usr/local/bin/rbash";
our $READLINK                  = "/usr/bin/readlink";
our $REBOOT                    = "/sbin/reboot";
our $RLOGIN                    = "/usr/bin/rlogin";
our $RM                        = "/bin/rm";
our $ROUTE                     = "/sbin/route";
our $RPCINFO                   = "/usr/sbin/rpcinfo";
our $RPM                       = "/bin/rpm";
our $RPMDB_VERIFY              = "/usr/lib/rpm/rpmdb_verify";
our $RPMQUERY                  = "/usr/bin/rpmquery";
our $RUNUSER                   = "/sbin/runuser";
our $SCP                       = "/usr/bin/scp";
our $SED                       = "/bin/sed";
our $SERVICE                   = "/sbin/service";
our $SFTPSERVER                = "/usr/libexec/openssh/sftp-server";
our $SMCRYPT                   = "/opt/nortel/base/bin/smCrypt";
our $SSH                       = "/usr/bin/ssh";
our $STTY                      = "/bin/stty";
our $SH                        = "/bin/sh";
our $SHUTDOWN                  = "/sbin/shutdown";
our $SU                        = "/bin/su";
our $SUDO                      = "/usr/bin/sudo";
our $SUDOEDIT                  = "/usr/bin/sudoedit";
our $SUDO_NORTEL               = "/opt/nortel/base/bin/sudo_nortel";
our $STAT                      = "/usr/bin/stat";
our $SYNC                      = "/bin/sync";
our $SYSBACKUP                 = "sysbackup";
our $SYSCTL                    = "/sbin/sysctl";
our $TAIL                      = "/usr/bin/tail";
our $TAR                       = "/bin/tar";
our $TEST                      = "/usr/bin/test";
our $TOUCH                     = "/bin/touch";
our $TTY                       = "/usr/bin/tty";
our $UMOUNT                    = "/bin/umount";
our $UNAME                     = "/bin/uname";
our $UNZIP                     = "/usr/bin/unzip";
our $WALL                      = "/usr/bin/wall";
our $USERADD                   = '/usr/sbin/useradd';
our $USERDEL                   = "/usr/sbin/userdel";
our $USERHELPER                = "/usr/sbin/userhelper";
our $USERMOD                   = "/usr/sbin/usermod";
our $WC                        = "/usr/bin/wc";
our $WHOAMI                    = "/usr/bin/whoami";
our $ZIP                       = "/usr/bin/zip";

#### Accounts
our $NORTEL_UID  = "500";
our $NORTEL_GID  = "500";
our $ROOT_UID    = "0";
our $NORTEL_ACCT = getpwuid($NORTEL_UID);
our ($ROOT_ACCT, undef, undef, $ROOT_GID) = getpwuid($ROOT_UID);
our $DEPLOY_ACCT = $NORTEL_ACCT;
our $NORTEL_GRP  = getgrgid($NORTEL_GID);
our $ROOT_GRP    = getgrgid($ROOT_GID);
our %GROUPS = (
               'nortel'        => $NORTEL_GID,
               'securityadmin' => 600,
               'logadmin'      => 601,
               'backupadmin'   => 602,
               'patchadmin'    => 603,
               'dbadmin'       => 604,
               'maintadmin'    => 605,
               'timeadmin'     => 606,
               'systemadmin'   => 607
              );

#XML_PATHS hash stores xml file names
our %XML_PATHS = ();

1;

__END__

=head1 NAME

target_common - common data for all base tools.

=head1 SYNOPSIS

use target_common;

=head1 LIMITATIONS

=head1 SEE ALSO

=cut
