###############################################################################
## @file 00OS_harden_linux_services.pm
# @brief This module disables inoperable Linux services
# @details This module is part of LinuxBase OS hardening and disables Linux
#          services which are not utilized by Linux Base
# @author Alexander Kolotov
# @version 1.00 - Initial version
# @version 1.01 - Two colons were removed before shared functions and variables.

## @class 00OS_harden_linux_services
package hardening::basic::00OS_harden_linux_services;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.01';

use lib "/opt/nortel/base/lib";

use strict;
use warnings;

use common_variables;
use hardening::hardening_common;
use baselog;
use base_system;

## @skipline
sub actions();
## @skipline
sub apply();

my %actions = ($H_IF_APPLY           => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_LB_INSTALL => {'sub'   => \&apply,
                                         'visible' => $FALSE},
               $H_IF_POST_PATCHING   => {'sub'   => \&apply,
                                         'visible' => $FALSE}
              );

my $DODitem = "OS hardening (services)";

my @services_for_removing = ("apmd", "atd", "autofs", "gpm", "ip6tables",
                             "kudzu", "lm_sensors", "mdmonitor", "mdmpd", 
                             "portmap", "sendmail", "snmpd", "vsftpd",
                             "xfs", "lvm2-monitor", "netconsole", "rdisc", "nscd");

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
   return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub apply()
# @brief Disables Linux services
# @details This method just disables generic Linux services listed in
#          \@services_for_removing - the service which are not utilized by
#          Linux Base
# @return TRUE - the option is hardened
# @return FALSE - some error happened
sub apply()
{
   my $retval = $TRUE;
   
   baselog($LOG_INFO, "Handling $DODitem...");

   foreach my $serv (@services_for_removing)
   {
      if (not do_system([$CHKCONFIG, "--del", $serv]))
      {
         baselog($LOG_ERROR, "Cannot remove the service '$serv'. It is necessary to check the service existence.");
         $retval = $FALSE;
      }
      else
      {
         baselog($LOG_INFO, "$DODitem: The service '$serv' is removed.");
      }
   }

   return $retval;
}

1;
__END__
