## @file header_xml.pm
# @brief this module incapsulates all operations with patch/su/sp header xml file
# @details none
# @author Sergey Urlin
# @date August 2008
# @version 1.00 - Initial version
# @version 1.10 - Modified due to changes in header.xml templates
# @version 1.12 - function "getHeaderValue" modified to return empty string if tag doesn't contain anything
#		- "ISS", "GEN_AVAILABLE_TIME" and "CATEGORY"  fields are removed from HEADER_TAGS.
# @version 1.13 - APPS field added to HEADER_TAGS, REPLACEABLE renamed as REPLACABLE, getApps() function added
# @version 1.23 - functions getMetaData and getChildTagsValue added
# @version 1.24 - Added MIN_RPM_VER and MUTEX tags in patch header.
# @version 1.25 - Fix for Enhancement CR Q02059485 - CS 1000 R6.0 Linux Patching Support of "REFRESH" SP Capability
#
# SP header.xml template:
#<?xml version="1.0" encoding="UTF-8" ?>
#<HEADER>
#     <GENERIC>LINUX</GENERIC>
#     <MACHINE>x86</MACHINE>
#     <TYPE>SP</TYPE>
#     <PRODUCT_RLS>6.00.00.00</PRODUCT_RLS>
#     <AUTHOR>Author name</AUTHOR>
#     <CREATION_TIME>time</CREATION_TIME>
#     <SUS>
#          <SNAME name="nortel-cs1000-vtrk-5.90.99-01.000.i386.ntl" />
#          <SNAME name="nortel-cs1000-gk-5.90.99-01.000.i386.ntl" />
#     </SUS>
#     <PATCHES>
#          <PNAME name="p12345_1.ntl" />
#     </PATCHES>
#     <PRE_SCRIPT>yes</PRE_SCRIPT>
#     <POST_SCRIPT>no</POST_SCRIPT>
#     <SPECINST>YES</SPECINST>
#     <MIRCC>yes|no</MIRCC>
#     <HOST_NAME>host.name.ca</HOST_NAME>
#     <DEPENDENT>
#          <DEPNAME name="nortel-cs1000-linuxbase-5.90.10-01.i386.ntl" />
#          <DEPNAME name="nortel-cs1000-java-5.90.10-01.i386.ntl" />
#          <DEPNAME name="nortel-cs1000-jboss-5.90.10-01.i386.ntl"/>
#     </DEPENDENT>
#     <OBSOLETE>
#          <OBSNAME name="p55555_1.ntl"/>
#     </OBSOLETE>
#     <ADDED_PACTHES>
#          <ADDNAME name="p12456_1.ntl"/>
#     </ ADDED_PACTHES >
#</HEADER>
#
# SU header.xml template:
#<?xml version="1.0" encoding="UTF-8" ?>
#<HEADER>
#        <GENERIC>LINUX</GENERIC>
#        <MACHINE>X86</MACHINE>
#        <TYPE>SU</TYPE>
#        <SUBTYPE>RPM</SUBTYPE>
#        <REMOVABLE>YES|NO</REMOVABLE>
#        <PATCH_NAME>nortel-cs1000-vtrk-6.00.00-00.i386.001</PATCH_NAME>
#        <AUTHOR>AUTHOR NAME</AUTHOR>
#        <CREATION_TIME>Fri Aug 8 15:38:48 EDT 2008</CREATION_TIME>
#        <RPM_NAME>nortel-cs1000-vtrk-6.00.00-00.i386.rpm</RPM_NAME>
#        <PRODUCT_RLS>
#                <RELEASE name="6.00.00" />
#        </PRODUCT_RLS>
#        <CR>Q01234567</CR>
#        <PRE_SCRIPT>YES|NO</PRE_SCRIPT>
#        <POST_SCRIPT>YES|NO</POST_SCRIPT>
#        <SPECINST>YES|NO</SPECINST>
#        <REBOOT>YES|NO</REBOOT>
#        <APP_START>
#                <APPLICATION id="2" name="nrsm" />
#                <APPLICATION id="3" name="isclient" />
#                <APPLICATION id="1" name="tps" />
#        </APP_START>
#        <APP_STOP>
#                <APPLICATION id="1" name="isclient" />
#                <APPLICATION id="3" name="nrsm" />
#                <APPLICATION id="2" name="tps" />
#        </APP_STOP>
#        <DEPENDENT>nortel-cs1000-linuxbase-6.00.00.rpm</DEPENDENT>
#        <OBSOLETE>
#                <OBSNAME id="2" name="privateCA-6.00.00.rpm" />
#                <OBSNAME id="1" name="cert_ipsec-6.00.00.rpm" />
#        </OBSOLETE>
#        <MEMBER_NAME>
#                <FILE id="1" name="nortel-cs1000-vtrk-6.00.00-00.i386.rpm" checksum="639892036cb25cf1997cc74546d5f7ff" />
#        </MEMBER_NAME>
#	 <META_DATA>
#		<RPM>
#                       <MEMBER id="1">
#                               <FILE id="1" name="<rpm file name>" />
#                       </MEMBER>
#               </RPM>
#	</META_DATA>
#</HEADER>
#
# patch header.xml template:
#<?xml version="1.0" encoding="UTF-8" ?>
#<HEADER>
#        <GENERIC>LINUX</GENERIC>
#        <MACHINE>x86</MACHINE>
#        <TYPE>PATCH</TYPE>
#        <SUBTYPE>BIN|JAR|WAR|FRU</SUBTYPE>
#        <REMOVABLE>YES</REMOVABLE>
#        <AUTHOR>Paul Li</AUTHOR>
#        <CREATION_TIME>Tue Sep  2 17:47:39 EDT 2008</CREATION_TIME>
#        <PATCH_NAME>p01war_1</PATCH_NAME>
#        <RPM_NAME>nortel-cs1000-vtrk-5.90.47-00.i386.rpm</RPM_NAME>
#        <PRODUCT_RLS />
#        <CR>Q01234567</CR>
#        <PRE_SCRIPT>YES|NO</PRE_SCRIPT>
#        <POST_SCRIPT>YES|NO</POST_SCRIPT>
#        <SPECINST>YES|NO</SPECINST>
#        <REBOOT>NO</REBOOT>
#        <APP_START>
#                <APPLICATION id="1" name="perl" />
#                <APPLICATION id="2" name="sps" />
#                <APPLICATION id="3" name="vtn" />
#        </APP_START>
#        <APP_STOP>
#                <APPLICATION id="1" name="perl" />
#                <APPLICATION id="2" name="sps" />
#                <APPLICATION id="3" name="vtn" />
#        </APP_STOP>
#        <DEPENDENT>p44444_1.ntl</DEPENDENT>
#        <OBSOLETE>
#                <OBSNAME id="1" name="p55555_1.ntl" />
#                <OBSNAME id="2" name="p66666_1.ntl" />
#                <OBSNAME id="3" name="p77777_2.ntl" />
#                <OBSNAME id="4" name="p88888_3.ntl" />
#        </OBSOLETE>
#        <MIN_RPM_VER>
#                <RPM_NAME id="1" name="nortel-cs1000-tps-6.00.00-00.i386"/>
#                <RPM_NAME id="2" name="nortel-cs1000-vtrk-6.00.00-00.i386"/>
#        </MIN_RPM_VER>
#        <MUTEX>
#                <FILE id ="1" name="p12345_1.ntl" />
#                <FILE id ="2" name="p12300_1.ntl" />
#        </MUTEX>
#        <MEMBER_NAME>
#                <FILE id="4" check_sum="21cde31ffc23739d1c8de0432739ba01" name="p01war_1_4.war" />
#                <FILE id="2" check_sum="7fb668593d7212c07174ec025f4aa9a5" name="p01war_1_2.tar" />
#                <FILE id="5" check_sum="nvujc885i4i969j67r874gh477858hbf" name="p01war_1_5.war" />
#                <FILE id="1" check_sum="uy347236gehdb7f7eu4h3jh487843348" name="p01war_1_1.war" />
#                <FILE id="3" check_sum="y3764748389hrhe84378y4y384y84y75" name="p01war_1_3.war" />
#        </MEMBER_NAME>
#	 <META_DATA>
#		 <FRU>
# 		 	<MEMBER id="1">
#         	 		<FILE id="1" name="full file path name" />
#				<FILE id="2" name="full file path name" />
#				..............
#			</MEMBER>
#			<MEMBER id="2">
#				<FILE id="1" name="full file path name" />
#				<FILE id="2" name="full file path name" />
#			</MEMBER>
#		</FRU>
#		<RPM>
#                       <MEMBER id="1">
#                               <FILE id="1" name="<rpm file name>" />
#                       </MEMBER>
#               </RPM>
#               <BIN>
#                       <MEMBER id="1">
#				<ORIG_PATH id="1" name="<full path name>" />
#				<REPL_NAME name="<lib name>" />
#                               <INCLUDE id="1" name="<function_name_1>" />
# 	                        <INCLUDE id="2" name="<function_name_2>" />
#                       </MEMBER>
#                       <MEMBER id="2">
#				................
#                       </MEMBER>
#	        </BIN>
#	        <JAR>
#                       <MEMBER id="1">
#                               <FILE id="1" name="<file/class path name>" />
#                               <FILE id="2" name="<file/class path name>" />
#                        </MEMBER>
#      		</JAR>
#            	<WAR>
#                        <MEMBER id="1">
#                               <FILE id="1" name="<file path name>" />
#                               <FILE id="2" name="<file path name>" />
#                	 </MEMBER>
#	       </WAR>
#	</META_DATA>
#</HEADER>

package patchutil::header_xml;
use lib "/opt/nortel/base/lib";

use warnings;
use strict;
use baselog;
use base_xml;
use common_variables;
use Exporter;

our $VERSION = "1.25";

our @ISA    = qw(Exporter);      # : public Exporter;
our @EXPORT = qw(&getHeaderValue
  &getAttribute
  &getDependency
  &getAppStart
  &getAppStop
  &getObsoletes
  &getReleases
  &getAttributesFromFile
  &getSPContent
  &getNamesSorted
  &getMetaData
  &getMinRpmVer
  &getMutex
  &getNames
  &determine_SP_type);

my $class = "patchutil::header_xml";

my %HEADER_TAGS = (
                   HEADER        => "HEADER",
                   GENERIC       => "GENERIC",
                   MACHINE       => "MACHINE",
                   TYPE          => "TYPE",
                   AUTHOR        => "AUTHOR",
                   CREATION_TIME => "CREATION_TIME",
                   PRE_SCRIPT    => "PRE_SCRIPT",
                   POST_SCRIPT   => "POST_SCRIPT",
                   MEMBER_NAME   => "MEMBER_NAME",
                   SPECINST      => "SPECINST",      # possible values YES or NO
                   APP_START     => "APP_START",
                   APP_STOP      => "APP_STOP",
                   DEPENDENT     => "DEPENDENT",
                   OBSOLETE      => "OBSOLETE",
                   APPLICATION   => "APPLICATION",
                   OBSNAME       => "OBSNAME",
                   PATCH_NAME    => "PATCH_NAME",
                   PRODUCT_RLS   => "PRODUCT_RLS",
                   REMOVABLE     => "REMOVABLE",
                   SUBTYPE       => "SUBTYPE",
                   RPM_NAME      => "RPM_NAME",
                   RELEASE       => "RELEASE",
                   FILE          => "FILE",
                   APPS          => "APPS",
                   CR            => "CR",
                   META_DATA     => "META_DATA",
                   RPM           => "RPM",
                   FRU           => "FRU",
                   BIN           => "BIN",
                   JAR           => "JAR",
                   WAR           => "WAR",
                   MEMBER        => "MEMBER",
                   ORIG_PATH     => "ORIG_PATH",
                   INCLUDE       => "INCLUDE",
                   REPL_NAME     => "REPL_NAME",
                   MIN_RPM_VER   => "MIN_RPM_VER",
                   MUTEX         => "MUTEX",
                  );

# service pack specific header tags
my %SP_HEADER_TAGS = (
                      PRODUCT_RLS => "PRODUCT_RLS",
                      SUS         => "SUS",
                      PATCHES     => "PATCHES",
                      SNAME       => "SNAME",
                      PNAME       => "PNAME",
                      MIRCC       => "MIRCC",
                      DEPNAME     => "DEPNAME",
                      ADDED_PATCH => "ADDED_PATCHES",
                      ADDNAME     => "ADDNAME",
                      HOST_NAME   => "HOST_NAME",
                     );
my %HEADER_ATTRIBUTES = (
                         name     => "name",
                         id       => "id",
                         checksum => "check_sum",
                        );

my $EMPTY_STR = "";

####################################################
# @method       new($header_file)
# @brief        Constructor of the object.
# @param        $header_file - xml file contained patch/SU/SP header info.
# @return       $self - reference to the object.
# @return       undef - if error occured
# @date         August 2008
####################################################
sub new ($)
{
   my $header_file = shift @_;
   my %self;
   if ($header_file eq "")
   {
      baselog($LOG_ERROR, "Header.xml file is not defined.");
      return undef;
   }
   my $header = base_xml->new($header_file);
   unless (defined $header)
   {
      baselog($LOG_ERROR, "Can't parse xml file: $header_file.");
      return undef;
   }
   $self{'header'} = $header;
   my $type = $header->findElements("$HEADER_TAGS{HEADER}/$HEADER_TAGS{TYPE}");
   unless (defined $type)
   {
      baselog($LOG_ERROR, "Can't retrieve TYPE by the path $HEADER_TAGS{HEADER}/$HEADER_TAGS{TYPE}");
      return undef;
   }
   my $txt = @{$type}[0]->textContent();
   if ($txt =~ /^(SU|SP|PATCH)$/)
   {

      if ($txt eq 'SP')
      {
         my $SP_type = determine_SP_type($header);
         if (not defined $SP_type)
         {
            baselog($LOG_ERROR, "Cannot determine SP type", $TRUE);
            return undef;
         }
         $self{'type'} = $SP_type;
      } ## end if ($txt eq 'SP')
      else
      {
         $self{'type'} = $txt;
      }
      baselog($LOG_INFO, "Got type: $self{'type'}.");
   } ## end if ($txt =~ /^(SU|SP|PATCH)$/...
   else
   {
      baselog($LOG_ERROR, "Type is not valid: $type. Only accepted SU, SP or PATCH.");
      return undef;
   }

   bless \%self, $class;
   return \%self;
} ## end sub new ($)

####################################################
# @method       DESTROY
# @brief        Object destructor.
# @param        $self - reference to the object
# @return       none.
# @date         August 2008
####################################################
sub DESTROY
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "$class object is not defined.");
   }
   elsif (defined $self->{'header'})
   {
      baselog($LOG_DEBUG, "Header object is defined.");
      undef $self->{'header'};

      #undef $self;
   } ## end elsif (defined $self->{'header'...
   baselog($LOG_DEBUG, "patch object has been destroyed");
   return;
} ## end sub DESTROY

#####################################################
# @method       getSPContent($self, $type)
# @brief        Retrieves Service Pack content: SUs or patches
#               from header xml file
# @param        $self - reference to the object.
# @param        $type - type to be retrieved (can be one of the following: SU, PATCHES)
# @return       array with requested information (Patches or SUs which are delivered with SP)
# @return       undef - if error occured.
# @date         August 2008
#####################################################
sub getSPContent($$)
{
   my ($self, $type) = @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object $class is undefined.");
      return undef;
   }
   elsif (not defined $self->{'header'})
   {
      baselog($LOG_ERROR, "XML header parser is not defined.");
      return undef;
   }
   if (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not $class object.");
      return undef;
   }
   if ($self->{'type'} !~ /^(SP|DELTA_SP|SEARCH_SP)$/)
   {
      baselog($LOG_ERROR, "SU/PATCH content is not defined for any type except SP.");
      return undef;
   }
   if ((not defined $type) or (($type ne 'SUS') and ($type ne 'PATCHES')))
   {
      baselog($LOG_ERROR, "Invalid type parameter: $type.");
      return undef;
   }

   my $tag_name = substr($type, 0, 1) . "NAME";
   return getNames($self, "$HEADER_TAGS{HEADER}/$SP_HEADER_TAGS{$type}/$SP_HEADER_TAGS{$tag_name}", "$HEADER_ATTRIBUTES{name}");
} ## end sub getSPContent($$)

#####################################################
# @method       getHeaderValue($self, $path)
# @brief        Retrieves tag by Xpath.
# @param        $self - reference to the object.
# @param        $path - Xpath to the tag
# @return       Header's text content.
# @return       undef - if error occured.
# @date         September 2008
#####################################################
sub getHeaderValue($$)
{
   my ($self, $path) = @_;
   if (not defined $path)
   {
      baselog($LOG_ERROR, "Invalid input parameter: Xpath.");
      return undef;
   }
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return undef;
   }
   elsif (not defined $self->{'header'})
   {
      baselog($LOG_ERROR, "XML header parser is not defined.");
      return undef;
   }
   if (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return undef;
   }
   my $header = $self->{'header'}->findElements($path);
   unless (defined $header)
   {
      baselog($LOG_ERROR, "Can't retrieve tag by Xpath: $path.");
      return undef;
   }
   if (scalar(@$header) == 0)
   {
      baselog($LOG_INFO, "No tags retrieved by Xpath($path) provided. Element is empty.");
      return undef;
   }
   elsif (scalar(@$header) > 1)
   {
      baselog($LOG_INFO, scalar(@{$header}) . " tags retrieved, check header.xml file for consistency. Taking first element.");
   }

   # $header - is reference to array of nodes
   if (@{$header}[0]->nodeType eq '3')
   {
      return @{$header}[0]->textContent();
   }
   elsif (@{$header}[0]->nodeType eq '1')
   {

      # we have tag with attributes - error condition
      if (@{$header}[0]->hasAttributes())
      {
         baselog($LOG_ERROR, "Node " . @{$header}[0]->nodeName . " has attributes. Please use another function (getAttributes).");
         return undef;
      }

      # complex tag, it has child nodes
      elsif (@{$header}[0]->hasChildNodes())
      {
         my $i = 0;
         $i++ foreach (@{$header}[0]->childNodes);
         if ($i > 1)
         {
            baselog($LOG_ERROR, @{$header}[0]->nodeName . " has more then one child nodes ($i). Please specify another Xpath.");
            return undef;
         }
         else
         {
            return @{$header}[0]->firstChild()->textContent();
         }
      } ## end elsif (@{$header}[0]->hasChildNodes...
        # this is incase if tag has no value
      else
      {
         baselog($LOG_INFO, @{$header}[0]->nodeName . " has no value, using empty string.");
         return $EMPTY_STR;
      }
   } ## end elsif (@{$header}[0]->nodeType...
        # unknown node type
   else
   {
      baselog($LOG_ERROR, "Unknown node type.");
      return undef;
   }
} ## end sub getHeaderValue($$)

#####################################################
# @method       getAttribute($self, $path; $attr)
# @brief        Retrieves tags attribute value by Xpath. only first attribute (key, value) is captured.
# @param        $self - reference to the object.
# @param        $path - Xpath to the tag
# @param        $attr - attribute name. if $attr is not specified return all available
#               attributes that were found for Xpath
# @return       reference to hash (key is attribute name and value is attribute value)
# @return       undef - if error occured.
# @date         September 2008
#####################################################
sub getAttribute($$;$)
{
   my ($self, $path, $attr) = @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return undef;
   }
   elsif (not defined $self->{'header'})
   {
      baselog($LOG_ERROR, "XML header parser is not defined.");
      return undef;
   }
   if (ref $self ne $class)
   {
      baselog($LOG_ERROR, "Object is not an instance of $class.");
      return undef;
   }
   my $header_attrs = $self->{'header'}->getAttrs($path);
   if (not defined $header_attrs)
   {
      baselog($LOG_ERROR, "Can't get attributes by Xpath: $path.");
      return undef;
   }
   else
   {
      if (not defined $attr)
      {
         baselog($LOG_INFO, "Attributes were not provided. Found " . keys(%$header_attrs) . " elements.");
         return $header_attrs;
      }
      else
      {
         my $retVal = {};
         foreach my $key (keys(%$header_attrs))
         {
            $retVal->{$key} = $header_attrs->{$key} if $key eq $attr;
         }
         return ((keys(%$retVal) == 0) ? undef: $retVal);
      } ## end else [ if (not defined $attr)
   } ## end else [ if (not defined $header_attrs...
} ## end sub getAttribute($$;$)

##########################################################
# @method       getMetaData($self)
# @brief        Gets information about patch/SU members (meta_data) files from
#               header xml file
# @param        $self - reference to the object.
# @return       undef - if error occured
# @return	list of members files
# @date         November 2008
##########################################################
sub getMetaData($)
{
   my $self      = shift @_;
   my @elem_list = ();
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return undef;
   }
   elsif (not defined $self->{'type'})
   {
      baselog($LOG_ERROR, "Type is undefined.");
      return undef;
   }
   if (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return undef;
   }
   if (($self->{'type'} eq "PATCH") or ($self->{'type'} eq "SU"))
   {
      my $subtype = getHeaderValue($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{SUBTYPE}");
      $subtype =~ tr/a-z/A-Z/;
      if ($subtype =~ /^FRU|JAR|WAR|RPM|BIN$/)
      {

         # retrieving members names and ids
         my %members = getNamesSorted($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{MEMBER_NAME}/$HEADER_TAGS{FILE}", "$HEADER_ATTRIBUTES{name}", "$HEADER_ATTRIBUTES{id}", $TRUE);
         my @keys_member = keys(%members);
         if ($subtype =~ /^BIN$/)
         {
            for (my $i = 1 ; $i < scalar(@keys_member) + 1 ; $i++)
            {
               baselog($LOG_INFO, "Getting elements for patch member: $members{$keys_member[$i-1]}, id: $keys_member[$i-1].");
               my %elem = ();
               my @orig_path = getChildTagsValue($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{META_DATA}/$HEADER_TAGS{$subtype}/$HEADER_TAGS{MEMBER}", "$HEADER_TAGS{ORIG_PATH}", $i);
               if (not scalar(@orig_path))
               {

                  # there can be only one 'ORIG_PATH' for the member
                  baselog($LOG_ERROR, "Patch member $members{$keys_member[$i-1]} doesn't have executable 'ORIG_PATH'");
                  return undef;
               } ## end if (not scalar(@orig_path...
               elsif (scalar(@orig_path) > 1)
               {
                  baselog($LOG_ERROR, "Patch member $members{$keys_member[$i-1]} has more then one executable (specified by ORIG_PATH): " . join(' ', @orig_path));
                  return undef;
               }
               my @repl_name = getChildTagsValue($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{META_DATA}/$HEADER_TAGS{$subtype}/$HEADER_TAGS{MEMBER}", "$HEADER_TAGS{REPL_NAME}", $i);
               if (not scalar(@repl_name))
               {

                  # there can be only one 'REPL_NAME' for the member
                  baselog($LOG_ERROR, "Patch member $members{$keys_member[$i-1]} doesn't have shared object 'REPL_NAME'");
                  return undef;
               } ## end if (not scalar(@repl_name...
               elsif (scalar(@orig_path) > 1)
               {
                  baselog($LOG_ERROR, "Patch member $members{$keys_member[$i-1]} has more then one library (specified by REPL_NAME): " . join(' ', @repl_name));
                  return undef;
               }
               my @includes = getChildTagsValue($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{META_DATA}/$HEADER_TAGS{$subtype}/$HEADER_TAGS{MEMBER}", "$HEADER_TAGS{INCLUDE}", $i);
               if (not scalar(@includes))
               {
                  baselog($LOG_ERROR, "Patch member $members{$keys_member[$i-1]} doesn't have included functions.");
                  return undef;
               }
               my $hash;
               $hash->{'exec'}  = $orig_path[0];
               $hash->{'repl'}  = $repl_name[0];
               $hash->{'funcs'} = [@includes];

               #$hash->{$orig_path[0] . "\t" . $repl_name[0]} = join("\t", @includes);
               push @elem_list, $hash;
            } ## end for (my $i = 1 ; $i < scalar...
         } ## end if ($subtype =~ /^BIN$/...
         else
         {
            for (my $i = 1 ; $i < scalar(@keys_member) + 1 ; $i++)
            {
               baselog($LOG_INFO, "Getting elements for patch member: $members{$keys_member[$i-1]}, id: $keys_member[$i-1].");
               my @elements = getChildTagsValue($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{META_DATA}/$HEADER_TAGS{$subtype}/$HEADER_TAGS{MEMBER}", "$HEADER_TAGS{FILE}", $i);
               baselog($LOG_INFO, "Using XPath:  $HEADER_TAGS{HEADER}/$HEADER_TAGS{META_DATA}/$HEADER_TAGS{$subtype}/$HEADER_TAGS{MEMBER}\[$i\]/$HEADER_TAGS{FILE}.");
               push @elem_list, @elements;

               #print ("element: $_\n") foreach (@elements);
            } ## end for (my $i = 1 ; $i < scalar...
         } ## end else [ if ($subtype =~ /^BIN$/...
      } ## end if ($subtype =~ /^FRU|JAR|WAR|RPM|BIN$/...
      else
      {
         baselog($LOG_ERROR, "Unsupported value in $HEADER_TAGS{SUBTYPE} tag: $subtype.");
         return undef;
      }
   } ## end if (($self->{'type'} eq...
   else
   {

      # tag META_DATA is not present for SP
      baselog($LOG_ERROR, "$HEADER_TAGS{META_DATA} tag is not supported for SP.");
      return undef;
   } ## end else [ if (($self->{'type'} eq...
   return @elem_list;
} ## end sub getMetaData($)

##########################################################
# @method       getChildTagsValue($self, $path, $filter, $id)
# @brief        Retrieves child tags attribute values. The parent tag
#		is located in xml by XPath.
# @param        $self - reference to the object.
# @param	$path - XPath to the parent tag
# @param	$filter - child tag node name.
# @param	$id - child tag attribute, which values need to be retrieved.
# @return       undef - if error occured
# @return       list with attribute values
# @date         November 2008
##########################################################
sub getChildTagsValue($$$$)
{
   my ($self, $path, $filter, $id) = @_;

   my @retList = ();

   # This is internal function so all checking done before
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return @retList;
   }
   elsif (not defined $self->{'header'})
   {
      baselog($LOG_ERROR, "XML header parser is not defined.");
      return @retList;
   }
   elsif (not defined $id)
   {
      baselog($LOG_ERROR, "ID of member is not defined.");
      return @retList;
   }
   elsif (not defined $filter)
   {
      baselog($LOG_ERROR, "Filter name is not specified.");
      return @retList;
   }
   my $entries = $self->{'header'}->findElements($path);
   unless (defined $entries)
   {
      baselog($LOG_ERROR, "Can't get attributes $path.");
      return @retList;
   }
   if (not scalar(@$entries))
   {
      baselog($LOG_INFO, "No entries found in header.xml.");
   }
   else
   {
      foreach my $tag (@$entries)
      {
         my @attrs = $tag->attributes();
         foreach my $attr (@attrs)
         {
            if (($attr->nodeName() eq $HEADER_ATTRIBUTES{'id'}) and ($attr->textContent() eq $id))
            {

               # retrieving all nodes ion matched tag
               my @nodes = $tag->getElementsByTagName($filter);
               if (scalar(@nodes))
               {
                  foreach my $node (@nodes)
                  {

                     # processing 'FILE' tag of specified member, no need to sort it by 'id'
                     my $file_name = $node->getAttribute("$HEADER_ATTRIBUTES{name}");
                     baselog($LOG_DEBUG, "Got filename: $file_name.");
                     push @retList, $file_name;
                  } ## end foreach my $node (@nodes)
               } ## end if (scalar(@nodes))
               else
               {
                  baselog($LOG_DEBUG, "No elements retrieved for for member.");
               }
            } ## end if (($attr->nodeName()...
         } ## end foreach my $attr (@attrs)
      } ## end foreach my $tag (@$entries)
   } ## end else [ if (not scalar(@$entries...
   return @retList;
} ## end sub getChildTagsValue($$$$)

##########################################################
# @method       getAppStart($self)
# @brief        Gets all application names to be started from the
#               header xml file
# @param        $self - reference to the object.
# @return       undef - if error occured
# @return       array with names of applications to be started
# @date         August 2008
##########################################################
sub getAppStart($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return undef;
   }
   elsif (not defined $self->{'type'})
   {
      baselog($LOG_ERROR, "Type is undefined.");
      return undef;
   }
   if (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return undef;
   }
   if (($self->{'type'} eq "PATCH") or ($self->{'type'} eq "SU"))
   {
      return getNamesSorted($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{APP_START}/$HEADER_TAGS{APPLICATION}", "$HEADER_ATTRIBUTES{name}", "$HEADER_ATTRIBUTES{id}", $FALSE);
   }
   else
   {
      baselog($LOG_ERROR, "Application start is not supported for SP.");
      return undef;
   }
} ## end sub getAppStart($)

##########################################################
# @method       getAppStop($self)
# @brief        Gets all application names to be stopped from the
#               header xml file
# @param        $self - reference to the object.
# @return       undef - if error occured
# @return       array with names of applications to be stopped
# @date         August 2008
##########################################################
sub getAppStop($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return undef;
   }
   elsif (not defined $self->{'type'})
   {
      baselog($LOG_ERROR, "Type is undefined.");
      return undef;
   }
   if (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return undef;
   }
   if (($self->{'type'} eq "PATCH") or ($self->{'type'} eq "SU"))
   {
      return getNamesSorted($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{APP_STOP}/$HEADER_TAGS{APPLICATION}", "$HEADER_ATTRIBUTES{name}", "$HEADER_ATTRIBUTES{id}", $FALSE);
   }
   else
   {
      baselog($LOG_ERROR, "Application stop is not supported for SP.");
      return undef;
   }
} ## end sub getAppStop($)

##########################################################
# @method       getApps($self)
# @brief        Gets all associated applications names from the
#               header xml file
# @param        $self - reference to the object.
# @return       undef - if error occured
# @return       array with names of applications
# @date         Oct 2008
##########################################################
sub getApps($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return undef;
   }
   elsif (not defined $self->{'type'})
   {
      baselog($LOG_ERROR, "Type is undefined.");
      return undef;
   }
   if (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return undef;
   }
   if (($self->{'type'} eq "PATCH") or ($self->{'type'} eq "SU"))
   {
      return getNamesSorted($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{APPS}/$HEADER_TAGS{APPLICATION}", "$HEADER_ATTRIBUTES{name}", "$HEADER_ATTRIBUTES{id}", $FALSE);
   }
   else
   {
      baselog($LOG_ERROR, "APPS tag is not supported for SP.");
      return undef;
   }
} ## end sub getApps($)
##########################################################
# @method       getMembers($self)
# @brief        Gets all patch/SU members names from the header xml file
# @param        $self - reference to the object.
# @return       undef - if error occured
# @return       array with patch/SU members name
# @date         August 2008
##########################################################
sub getMembers($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return undef;
   }
   elsif (not defined $self->{'type'})
   {
      baselog($LOG_ERROR, "Type is undefined.");
      return undef;
   }
   if (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return undef;
   }
   if (($self->{'type'} eq "PATCH") or ($self->{'type'} eq "SU"))
   {
      return getNamesSorted($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{MEMBER_NAME}/$HEADER_TAGS{FILE}", "$HEADER_ATTRIBUTES{name}", "$HEADER_ATTRIBUTES{id}", $FALSE);
   }
   else
   {
      baselog($LOG_ERROR, "SP doesn't have members. Please use getSPContent function.");
      return undef;
   }
} ## end sub getMembers($)

##########################################################
# @method       getMD5Sums($self; $name, $sort)
# @brief        Gets all patch/SU members check sums from the header xml file
# @param        $self - reference to the object.
# @return       undef - if error occured
# @return       array with names of applications to be stopped
# @date         August 2008
##########################################################
sub getMD5Sums($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return undef;
   }
   elsif (not defined $self->{'type'})
   {
      baselog($LOG_ERROR, "Type is undefined.");
      return undef;
   }
   if (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return undef;
   }
   if (($self->{'type'} eq "PATCH") or ($self->{'type'} eq "SU"))
   {
      return getNamesSorted($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{MEMBER_NAME}/$HEADER_TAGS{FILE}", "$HEADER_ATTRIBUTES{checksum}", "$HEADER_ATTRIBUTES{id}", $FALSE);
   }
   else
   {
      baselog($LOG_ERROR, "SP doesn't have members tag. Please use getSPContent function.");
      return undef;
   }
} ## end sub getMD5Sums($)

##########################################################
# @method       getDependency($self)
# @brief        Gets dependency tag text from the header xml file
# @param        $self - reference to the object.
# @return       undef - if error occured
# @return       text content of the dependent tag
# @date         August 2008
##########################################################
sub getDependency($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return undef;
   }
   elsif (not defined $self->{'type'})
   {
      baselog($LOG_ERROR, "Type is undefined.");
      return undef;
   }
   if (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return undef;
   }
   if (($self->{'type'} eq "PATCH") or ($self->{'type'} eq "SU"))
   {
      return getHeaderValue($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{DEPENDENT}");
   }
   elsif ($self->{'type'} =~ /^(SP|DELTA_SP|SEARCH_SP)$/)
   {
      return getNames($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{DEPENDENT}/$SP_HEADER_TAGS{DEPNAME}", "$HEADER_ATTRIBUTES{name}");
   }
} ## end sub getDependency($)

##########################################################
# @method       getObsoletes($self)
# @brief        Gets obsolete tags text from the header xml file
# @param        $self - reference to the object.
# @return       undef - if error occured
# @return       array with obsolete patches/sus names
# @date         September 2008
##########################################################
sub getObsoletes($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return undef;
   }
   elsif (not defined $self->{'type'})
   {
      baselog($LOG_ERROR, "Type is undefined.");
      return undef;
   }
   if (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return undef;
   }
   baselog($LOG_INFO, "Retrieving obsoletes for $self->{type}.");
   if ($self->{'type'} =~ /^(SP|DELTA_SP|SEARCH_SP)$/)
   {
      return getNames($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{OBSOLETE}/$HEADER_TAGS{OBSNAME}", "$HEADER_ATTRIBUTES{name}");
   }
   elsif (($self->{'type'} eq 'SU') or ($self->{'type'} eq 'PATCH'))
   {
      return getNamesSorted($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{OBSOLETE}/$HEADER_TAGS{OBSNAME}", "$HEADER_ATTRIBUTES{name}", "$HEADER_ATTRIBUTES{id}", $FALSE);
   }
   else
   {
      baselog($LOG_ERROR, "Unknown type provided: $self->{type}. Type can be one of the following: SP, DELTA_SP, SEARCH_SP, SU, PATCH.");
      return undef;
   }
} ## end sub getObsoletes($)

##########################################################
# @method       getReleases($self)
# @brief        Gets releases for which SU can be applied
#               from the header xml file. (applicable only for SU)
# @param        $self - reference to the object.
# @return       undef - if error occured
# @return       array with obsolete patches/sus names
# @date         August 2008
##########################################################
sub getReleases($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return undef;
   }
   elsif (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return undef;
   }
   elsif (not defined $self->{'type'})
   {
      baselog($LOG_ERROR, "Type is undefined.");
      return undef;
   }
   if ($self->{'type'} =~ /^PATCH|SU$/)
   {
      return getNames($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{PRODUCT_RLS}/$HEADER_TAGS{RELEASE}", "$HEADER_ATTRIBUTES{name}");
   }
   elsif ($self->{'type'} =~ /^(SP|DELTA_SP|SEARCH_SP)$/)
   {

      # only one release
      return getHeaderValue($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{PRODUCT_RLS}");
   } ## end elsif ($self->{'type'} =~...
   else
   {
      baselog($LOG_ERROR, "Unknown type provided: $self->{type}. Type can be one of the following: SP, DELTA_SP, SEARCH_SP, SU, PATCH.");
      return undef;
   }
} ## end sub getReleases($)

##########################################################
# @method       getNames($self, $path; $name)
# @brief        Returns all tag values for the provided attribute
#               For example if xml file contains:
#               <PRODUCT_RLS>
#                       <RELEASE name="name1" />
#                       <RELEASE name="name2" />
#                       <RELEASE name="name3" />
#               </PRODUCT_RLS>
#               then array contained name1, name2, name3 will be returned.
# @param        $self - reference to the object.
# @param        $path - Xpath to the tag.
# @param        $name - attribute key, if is not provided then default value 'name' is used
# @return       undef - if error occured
# @return       array with attribute values.
# @date         September 2008
##########################################################
sub getNames($$;$)
{
   my ($self, $path, $name) = @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return undef;
   }
   elsif (not defined $self->{'header'})
   {
      baselog($LOG_ERROR, "XML header parser is not defined.");
      return undef;
   }

   my $entries = $self->{'header'}->getAttrsforallElements($path);
   unless (defined $entries)
   {
      baselog($LOG_ERROR, "Can't get attributes $path.");
      return undef;
   }
   unless (defined $name)
   {
      baselog($LOG_INFO, "Using default search parameter: 'name'.");
      $name = "name";
   }
   my @names;
   if (scalar(@$entries) == 0)
   {
      baselog($LOG_INFO, "No entries found in header.xml.");
   }
   else
   {
      foreach my $val (@$entries)
      {
         while (my ($key, $value) = each(%$val))
         {
            push @names, $value if ($key eq $name);
         }

      } ## end foreach my $val (@$entries)
   } ## end else [ if (scalar(@$entries) ...
   return @names;
} ## end sub getNames($$;$)

##########################################################
# @method       getAttributesFromFile($self)
# @brief        Gets patch/su attributes (dependency, obsoletes, app_stop,
#               app_start from the header xml file
# @param        $self - reference to the object.
# @return       undef - if error occured
# @return       hash with references to array (includes reference to array of
#               dependencies (acctually dependency can be only one, it's
#               done for consistence), obsoletes, application names to be
#               started and stopped.
# @date         August 2008
##########################################################
sub getAttributesFromFile($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return undef;
   }
   elsif (not defined $self->{'header'})
   {
      baselog($LOG_ERROR, "XML header parser is not defined.");
      return undef;
   }
   unless (defined $self->{'type'})
   {
      baselog($LOG_ERROR, "Type is undefined.");
      return undef;
   }
   if ($self->{'type'} =~ /^(SP|DELTA_SP|SEARCH_SP)$/)
   {
      return undef;
   }
   my %attrs = (
                DEPENDENCY  => [getDependency($self)],
                OBSOLETES   => [getObsoletes($self)],
                APP_START   => [getAppStart($self)],
                APP_STOP    => [getAppStop($self)],
                MIN_RPM_VER => [getMinRpmVer($self)],
                MUTEX       => [getMutex($self)],
               );
   return \%attrs;
} ## end sub getAttributesFromFile($)

##########################################################
# @method       getNamesSorted($self, $path; $name, $sort)
# @brief        Returns all tag values for the provided attribute sorted by another attribute
#               For example if xml file contains:
#               <PRODUCT_RLS>
#                       <RELEASE id="1" name="name1" />
#                       <RELEASE id="3" name="name2" />
#                       <RELEASE id="2" name="name3" />
#               </PRODUCT_RLS>
#               then array contained name1, name3, name2 will be returned.
# @param        $self - reference to the object.
# @param        $path - Xpath to the tag.
# @param        $name - attribute key which values to return, if is not provided then default value 'name' is used
# @param        $sort - sorting attribute key, if is not provided then default value 'id' is used
# @param	$need_hash - optional variable if specified and set to TRUE, then hash which keys are sorting attribute
#		and values are returning attribute will be returned. Otherwise it returns array.
# @return       undef - if error occured
# @return       array with attribute values.
# @return 	hash with pairs attribute key / attribute value
# @date         September 2008
##########################################################
sub getNamesSorted($$;$$$)
{
   my ($self, $path, $name, $sort, $need_hash) = @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object $class is not defined.");
      return undef;
   }
   elsif (not defined $self->{'header'})
   {
      baselog($LOG_ERROR, "XML header parser is not defined.");
      return undef;
   }
   unless (ref $self eq $class)
   {
      baselog($LOG_ERROR, "Invalid argument $self. Instance of $class expected.");
      return undef;
   }
   my $entries = $self->{'header'}->getAttrsforallElements($path);
   unless (defined $entries)
   {
      baselog($LOG_ERROR, "Can't get attributes $path.");
      return undef;
   }
   unless (defined $name)
   {
      $name = $HEADER_ATTRIBUTES{'name'};
      baselog($LOG_DEBUG, "Using default search parameter: '$name'.");
   }
   unless (defined $sort)
   {
      $sort = $HEADER_ATTRIBUTES{'id'};
      baselog($LOG_DEBUG, "Using default sort parameter: '$sort'.");
   }
   if (not defined $need_hash)
   {
      baselog($LOG_DEBUG, "Will return list of names.");
      $need_hash = $FALSE;
   }
   elsif ($need_hash)
   {
      baselog($LOG_DEBUG, "Will return hash of names/ids.");
   }
   my %hash_values;
   my ($pushed_key, $pushed_value) = ("", "");
   if (scalar(@$entries) == 0)
   {
      baselog($LOG_INFO, "No entries found in header.xml.");
   }
   else
   {
      foreach my $val (@$entries)
      {
         while (my ($key, $value) = each(%$val))
         {
            $pushed_key   = $value if $key eq $sort;
            $pushed_value = $value if $key eq $name;
         }
         if ($pushed_key ne "" and $pushed_value ne "")
         {
            $hash_values{$pushed_key} = $pushed_value;
            ($pushed_key, $pushed_value) = ("", "");
         }
      } ## end foreach my $val (@$entries)
   } ## end else [ if (scalar(@$entries) ...
   if (not $need_hash)
   {
      my @names;
      push(@names, $hash_values{$_}) foreach (sort (keys %hash_values));
      return @names;
   } ## end if (not $need_hash)
   else
   {
      my %names;
      $names{$_} = $hash_values{$_} foreach (sort (keys %hash_values));
      return %names;
   } ## end else [ if (not $need_hash)
} ## end sub getNamesSorted($$;$$$)

##########################################################
# @method       getMinRpmVer($self)
# @brief        Gets all rpm required as minimum of
#               rpm installed in the system from the
#               header xml file.
# @param        $self - reference to the object.
# @return       undef - if error occured
# @return       array with names of rpms
# @date         Feb 2009
##########################################################
sub getMinRpmVer($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return undef;
   }
   elsif (not defined $self->{'type'})
   {
      baselog($LOG_ERROR, "Type is undefined.");
      return undef;
   }
   if (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return undef;
   }
   if ($self->{'type'} eq "PATCH")
   {
      return getNamesSorted($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{MIN_RPM_VER}/$HEADER_TAGS{RPM_NAME}", "$HEADER_ATTRIBUTES{name}", "$HEADER_ATTRIBUTES{id}", $FALSE);
   }
   else
   {
      baselog($LOG_ERROR, "Minumum RPM version only supported for patch types.");
      return undef;
   }
} ## end sub getMinRpmVer($)

##########################################################
# @method       getMutex($self)
# @brief        Gets all patches that are mutually
#               exclusive with current patch from the
#               header xml file.
# @param        $self - reference to the object.
# @return       undef - if error occured
# @return       array with patches file names.
# @date         Feb 2009
##########################################################
sub getMutex($)
{
   my $self = shift @_;
   if (not defined $self)
   {
      baselog($LOG_ERROR, "Object is undefined.");
      return undef;
   }
   elsif (not defined $self->{'type'})
   {
      baselog($LOG_ERROR, "Type is undefined.");
      return undef;
   }
   if (ref $self ne $class)
   {
      baselog($LOG_ERROR, "It is not an instance of $class.");
      return undef;
   }
   if ($self->{'type'} eq "PATCH")
   {
      return getNamesSorted($self, "$HEADER_TAGS{HEADER}/$HEADER_TAGS{MUTEX}/$HEADER_TAGS{FILE}", "$HEADER_ATTRIBUTES{name}", "$HEADER_ATTRIBUTES{id}", $FALSE);
   }
   else
   {
      baselog($LOG_ERROR, "Mutex only supported for patch types.");
      return undef;
   }
} ## end sub getMutex($)

##########################################################
# @method       determine_SP_type($header)
# @brief        Determine SP type by MIRCC and HOST_NAME
#               tag values
# @details      1. SP - <MIRCC>no</MIRCC>
#               2. DELTA_SP - <MIRCC>yes</MIRCC>,
#                             <HOST_NAME>linux.com</HOST_NAME>
#                             (i.e. <HOST_NAME> tag is not blank)
#               3. SEARCH_SP - <MIRCC>yes</MIRCC>,
#                              <HOST_NAME></HOST_NAME>
#                              (i.e. <HOST_NAME> tag is blank)
# @param        $header - reference to the header object.
# @return       $type - SP type (SP, DELTA_SP, SEARCH_SP)
# @return       undef - if error occured
# @author       Michael Samokhin
# @date         September 2009
##########################################################
sub determine_SP_type($)
{
   my $header = shift @_;
   if (not defined $header)
   {
      baselog($LOG_ERROR, "Header object is undefined.");
      return undef;
   }

   my ($MIRCC, $HOST_NAME, $type) = (undef, undef, undef);

   $MIRCC = $header->findElements("$HEADER_TAGS{HEADER}/$SP_HEADER_TAGS{MIRCC}");
   if (not defined $MIRCC)
   {
      baselog($LOG_ERROR, "Can't retrieve MIRCC value by the path $HEADER_TAGS{HEADER}/$SP_HEADER_TAGS{MIRCC}");
      return undef;
   }
   $MIRCC = @{$MIRCC}[0]->textContent();
   if ($MIRCC !~ /^(yes|no)$/i)
   {
      baselog($LOG_ERROR, "MIRCC tag must have 'yes' or 'no' value.");
      return undef;
   }
   $HOST_NAME = $header->findElements("$HEADER_TAGS{HEADER}/$SP_HEADER_TAGS{HOST_NAME}");
   if (not defined $HOST_NAME)
   {
      baselog($LOG_ERROR, "Can't retrieve HOST NAME by the path $HEADER_TAGS{HEADER}/$SP_HEADER_TAGS{HOST_NAME}");
      return undef;
   }
   $HOST_NAME = @{$HOST_NAME}[0]->textContent();
   if ($MIRCC =~ /^no$/i)
   {
      $type = 'SP';
   }
   elsif ($HOST_NAME ne $EMPTY_STR)
   {
      $type = 'DELTA_SP';
   }
   else
   {
      $type = 'SEARCH_SP';
   }

   return $type;
} ## end sub determine_SP_type($)

1;

