## @file patches_target.pm
# @brief This module contains common variables and functions required for
#        Linux Base patching facility.
# @details none
# @author Alex Arkhipov
# @author Evgeny Lazarev
# @author Vladimir Ivanov
# @date May 2008
# @version 1.00 - Initial version
# @version 1.01 - some patch function added
# @version 1.02 - $PATCHDIR and $PATCHEXT variables added
# @version 1.05 - function 'check_pkg' is modified in order to support
#   the Base Applications patching.
# @version 1.06 - Digest module added to calculate the md5 checksum
# @version 1.07 - Two colons were removed before shared functions and
#   variables. Shared functions and variables were exported.
# @version 1.08 - Modified under the perlcheck warnings cleanup activity.
# @version 1.11 - hash %PATCH_FIELDS is changed, new values are added.
# @version 1.12 - Add $SRV_PACK_DIR and $REVERT directories.
# @version 1.13 - ISS, GEN_AVAL_TIME, CATEGORY are removed from hash %PATCH_FIELDS
# @version 1.14 - APPS, FILENAME, PATCHPATH added to %PATCH_FIELDS. check_pkg removed.
# @version 1.15 - Move get_full_patch_file function to patch.pm
# @version 1.16 - Add declaration for spstat command.
# @version 1.17 - PATCH_FIELDS hash modified
# @version 1.18 - HEADER_FIELDS hash added
# @version 1.19 - Added MIN_RPM_VER and MUTEX to %PATCH_FIELDS.
# @version 1.20 - Add $PI_CONTROL_DIR.
# @version 1.21 - New function get_fault_sp_str() added.
# @note
package patchutil::patches_target;
use lib "/opt/nortel/base/lib";

use warnings;
use strict;
use common_variables;
use common_functions;
use install_common;
use baselog;
use patchutil::patches_public;
use ifxml;
use Digest;

use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&get_patch_info &get_patch_attr &get_patch_md5sum &check_pkg  &get_fault_sp_str
  $SPECINS_DIR $SRV_PACK_DIR $REVERT_DIR $SPLOAD_DIR $SPINS_DIR $SP_REVERT_DIR $CPLIST_FILE $SPLOAD_FILE $SPINS_FILE
  $SPLOAD_INFO_FILE $SPINS_INFO_FILE $HANDLES $HANDLES_ELEMENTS $BASE_RELEASE_NAME $GRUBCONF $MODULE_BASE_DIR
  $UPDATE_GRUB $RPM_COMMAND $MERCURY_SIGN $KERNEL_NUMBERS $BOOT_TIMEOUT $OUT_SERV $IN_SERV
  $IN_SERV_REBOOT $OUT_SERV_REBOOT $ALL_PATCHES $ALL_IN_SERVICE $ALL_OUT_SERVICE
  %PATCH_FIELDS %HEADER_FIELDS $MODINFO @ORIG_APP_RPMS $PATCHDIR $PATCHEXT $KERNEL_RPM $NORTEL_RPM $SYSTEM_RPM
  $PSTAT $PLIS $ISSP $PLOAD $PINS $POOS $POUT $SPLOAD $SPINS $SPOUT $PI_CONTROL_DIR $PM_SEPARATOR $pOK $pERROR $PATCH_str
  $STATUS_str
  );

our $VERSION = 1.21;

our $PATCHDIR = '/var/opt/nortel/patch';
our $PATCHEXT = 'ntl';

our $SPECINS_DIR   = $PATCHDIR . "/.SPECINS";     #specins dir
our $SRV_PACK_DIR  = $PATCHDIR . "/sp";           #service pack dir
our $REVERT_DIR    = $PATCHDIR . "/.revert";      #revert dir
our $SPLOAD_DIR    = $SRV_PACK_DIR . "/.spload";  #spload dir
our $SPINS_DIR     = $SRV_PACK_DIR . "/.spins";   #spins dir
our $SP_REVERT_DIR = $SRV_PACK_DIR . "/.revert";  #sp/revert dir

our $HANDLES           = $PATCHDIR . "/.handles";           #patch handles
our $HANDLES_ELEMENTS  = $PATCHDIR . "/.handles_elements";  #elements DB
our $BASE_RELEASE_NAME = "nortel-cs1000-linuxbase";         #rpm package name for version checking

our $PI_CONTROL_DIR = "var/opt/nortel/piControl/";          # used to verify member of pi-control patches
our $GRUBCONF       = "/boot/grub/grub.conf";

#PM interface
our $PM_SEPARATOR = "-------------------------------------------------";
our $pOK          = "OK";
our $pERROR       = "ERROR";
our $PATCH_str    = "!PATCH:";
our $STATUS_str   = "!STATUS:";

#base directory for modules
our $MODULE_BASE_DIR = "/opt/nortel/base/patchutil/binary";

our $UPDATE_GRUB = add_sudo_cmd() . "/opt/nortel/base/patchutil/vulnerability/update_grub";
our $RPM_COMMAND = add_sudo_cmd() . $RPM;

#filename , which will be created when the mercury module is upgraded
our $MERCURY_SIGN = "/var/tmp/mercury";
our $SPLOAD_FILE  = $SPLOAD_DIR . "/spload.txt";  # contains a list of messages for all patches and
                                                  # SUS which were successfully loaded during SPLOAD

our $CPLIST_FILE = $SPLOAD_DIR . "/cplist.txt";   # contains all patches and SUS copied to the
                                                  # $PATCHDIR from $SPLOAD_DIR

our $SPINS_FILE = $SPINS_DIR . "/spins.txt";      # contains a list of messages with the status
                                                  # result of putting in srv every patch and su
                                                  # included in a SP during a spins command

our $SPLOAD_INFO_FILE = $SPLOAD_DIR . "/spload_info.txt";  # contains results from spload
our $SPINS_INFO_FILE  = $SPINS_DIR . "/spins_info.txt";    # contains results from spins

our $KERNEL_NUMBERS = 3;                                   # number of kernels in grub
our $BOOT_TIMEOUT   = 0;                                   #seconds. Time out until loading default kernel

#predefined constants. Patch attribute
our $OUT_SERV        = 0;                                  # out of service
our $IN_SERV         = 1;                                  # in service
our $IN_SERV_REBOOT  = 2;                                  # in service after reboot
our $OUT_SERV_REBOOT = 3;                                  # out of service after reboot

#predefined constants which is used for searching
our $ALL_PATCHES     = -1;                                 # all patches (is service and out of service)
our $ALL_IN_SERVICE  = -2;                                 # all patches in service (including in
                                                           # service after reboot)
our $ALL_OUT_SERVICE = -3;                                 # all patches out of service

#RPM types
our $KERNEL_RPM = 1;                                       # kernel SU
our $NORTEL_RPM = 2;                                       # Nortel baseApp or App
our $SYSTEM_RPM = 3;                                       # System RPM, no local backup.

#CLI command constants
our $PSTAT  = 'pstat';
our $PLIS   = 'plis';
our $ISSP   = 'issp';
our $PLOAD  = 'pload';
our $PINS   = 'pins';
our $POOS   = 'poos';
our $POUT   = 'pout';
our $SPLOAD = 'spload';
our $SPINS  = 'spins';
our $SPOUT  = 'spout';

#Hash described fields for one patch handle
our %PATCH_FIELDS = (
                     'GENERIC',      "0",
                     'MACHINE',      "1",
                     'TYPE',         "2",
                     'SUBTYPE',      "3",
                     'REMOVABLE',    "4",
                     'AUTHOR',       "5",
                     'CREATE_TIME',  "6",
                     'RPM_NAME',     "7",
                     'PATCHNAME',    "8",
                     'PRE_SCRIPT',   "9",
                     'POST_SCRIPT',  "10",
                     'SPECINST',     "11",
                     'REBOOT',       "12",
                     'CR',           "13",
                     'PATCHPATH',    "14",
                     'FILENAME',     "15",
                     'PRODUCT_RLS',  "_prod_rls",
                     'APPS',         "_applications",
                     'APP_START',    "_appstart",
                     'APP_STOP',     "_appstop",
                     'DEPENDENT',    "_deps",
                     'OBSOLETES',    "_obsoletes",
                     'MEM_INFO',     "_mem_info",
                     'MEM_MD5',      "_mem_md5",
                     'META_DATA',    "_meta_data",
                     'EXEC',         "_exec",
                     'STATUS',       "_in-service",
                     'PLOAD',        "_pload_time",
                     'PINS',         "_pins_time",
                     'POOS',         "_poos_time",
                     'MIN_RPM_VER',  "_min_rpm_ver",
                     'MUTEX',        "_mutex",
                    );

our %HEADER_FIELDS = (
                      'PATCH_NAME',     "HEADER/PATCH_NAME",
                      'GENERIC',        "HEADER/GENERIC",
                      'MACHINE',        "HEADER/MACHINE",
                      'REMOVABLE',      "HEADER/REMOVABLE",
                      'AUTHOR',         "HEADER/AUTHOR",
                      'CREATE_TIME',    "HEADER/CREATION_TIME",
                      'PRE_SCRIPT',     "HEADER/PRE_SCRIPT",
                      'POST_SCRIPT',    "HEADER/POST_SCRIPT",
                      'SPECINST',       "HEADER/SPECINST",
                      'REBOOT',         "HEADER/REBOOT",
                      'RPM_NAME',       "HEADER/RPM_NAME",
                      'SUBTYPE',        "HEADER/SUBTYPE",
                      'CR',             "HEADER/CR",
                     );

our $MODINFO = "/sbin/modinfo";  #path to modinfo utility

our @ORIG_APP_RPMS = ();
####################
# Shared functions #
####################

## @method get_patch_md5sum($patchfile)
# @brief   calculate md5 checksum for the given patch file.
# @details
# @param   $patchfile - path to the patch file.
# @return $md5sum - MD5 checksum for the given patch file
# @return undef   - an error occurred
# @author Vladimir Ivanov
# @date    Aug 2008
# @note    None.
sub get_patch_md5sum($)
{
   my $patchfile = shift @_;
   my $md5sum    = undef;

   unless (-e $patchfile and -r $patchfile)
   {
      baselog($LOG_ERROR, "File '$patchfile' does not exist or is not readable.");
      return $md5sum;
   }

   my $fd;
   unless (open($fd, '<', $patchfile))
   {
      baselog($LOG_ERROR, "Cannot open file '$patchfile'");
      return $md5sum;
   }

   #Create new Digest object
   my $digest = Digest->new("MD5");

   #Calculate the digest for the file
   $digest->addfile($fd);
   close $fd;

   $md5sum = $digest->hexdigest;
   baselog($LOG_INFO, "File $patchfile md5 checksum is $md5sum");

   #delete the Digest object
   undef $digest;

   return $md5sum;
} ## end sub get_patch_md5sum($)

## @method check_pkg($backupdir, @files)
# @brief   check the RPM package type.
# @details
# @param   $backupdir - directory where backed up RPM is situated.
# @param   @files - RPM package filenames
# @return 0 (FALSE) - not RPM package
# @return 1 - kernel patch rpm's
# @return 2 - update rpm's
# @return 3 - rpm patch (application) rpm's , @orig_app_rpms - Full names of
#             original rpm packages.
# @return
# @author Alex Arkhipov
# @author Evgeny Lazarev
# @date    May 2008
# @note    None.
sub check_pkg(@)
{
   my (@files, @app_rpms, $list, $dir, $app_name, $backupdir);
   our @orig_app_rpms = ();
   ($backupdir, @files) = (@_);
   chomp(@files);

   #check RPM package using FILE command
   return $FALSE
     if system("for i in @files ; do [ \`file \$i | awk \'{print \$2}\'` == \'RPM\' ] || exit 1 ; done ") >> 8;

   #check RPM package using rpm command and get package names
   @files = `for i in @files ; do $RPM_COMMAND -qp --nosignature --qf \"\%{Name}\\n\" \$i ; done `;
   return $FALSE if $? >> 8;
   chomp(@files);

   #check kernel update
   foreach (@files)
   {
      if (/^kernel(-smp)?$/) {return $KERNEL_RPM;}  #kernel RPMs
   }

   #get base application rpm package names
   $list = join(',', @files);
   $list = "{$list}" if scalar(@files) > 1;
   @app_rpms = `for i in $backupdir/$list*.rpm ; do $RPM_COMMAND -qp --nosignature --qf '%{Name}\\n' \$i && echo \$i ;done`;
   chomp(@app_rpms);

   @ORIG_APP_RPMS = ();

   #check app rpm pkg.
   foreach (@files)
   {
      if ((scalar(@app_rpms) > 1) && (defined({@app_rpms}->{"$_"})))  # non-app RPM
      {
         push(@ORIG_APP_RPMS, {@app_rpms}->{"$_"});
         return $NORTEL_RPM;                                          #SU
      }
   } ## end foreach (@files)

   @app_rpms = `for i in $REPACKAGEDIR/$list*.rpm ; do $RPM_COMMAND -qp --nosignature --qf '%{Name}\\n' \$i && echo \$i ;done`;
   chomp(@app_rpms);

   #check rpm repackage directory. There should be a backup in the REPACKAGEDIR if you're doing a poos.
   #No backup will be present on a pins.
   foreach (@files)
   {
      unless ((scalar(@app_rpms) > 1) && (defined({@app_rpms}->{"$_"})))  # non-app RPM
      {
         @ORIG_APP_RPMS = ();
         return $SYSTEM_RPM;                                              #pins of system rpm SU
      }
      push(@ORIG_APP_RPMS, {@app_rpms}->{"$_"});
   } ## end foreach (@files)
   return $SYSTEM_RPM;                                                    #poos of system rpm SU

} ## end sub check_pkg(@)

########
#Description: get information about patch and return those fields of the
#             information which are specified in the passed hash
#INPUT:  $_[0] - reference to database hash
#        $_[1] - handle of patch
#        $_[2] - reference to resulting hash (keys which will be
#                filled by this subroutine should be already defined)
#OUTPUT: $TRUE (and $! = 0) - resulting hash is filled
#        $FALSE - some error happens:
#        if $! = 1 - there is no patch with such handle
#           $! = 2 - there is no such attribute
########
sub get_patch_info(@)
{
   my @keys_tmp;
   my @patch_desc;
   my $field;
   my $dbkey;
   my @meminfo;

   my %db     = %{$_[0]};
   my $handle = $_[1];
   my $res    = $_[2];

   # Check whether the patch exists
   unless (exists($db{$handle}))
   {
      baselog($LOG_WARNING, "There is no patch with handle $handle");
      $! = 1;
      return $FALSE;
   } ## end unless (exists($db{$handle...

   # Parse patch descriptor
   @patch_desc = split /\t/, $db{$handle};

   # Find which keys should be filled
   @keys_tmp = keys(%{$res});
   foreach my $key (@keys_tmp)
   {
      unless (exists($PATCH_FIELDS{$key}))
      {
         baselog($LOG_WARNING, "There is no attribute: $key");
         $! = 2;
         return $FALSE;
      } ## end unless (exists($PATCH_FIELDS...

      # Fill key from patch descriptor
      if ($PATCH_FIELDS{$key} =~ /^\d+$/)
      {
         $field = $PATCH_FIELDS{$key};
         $res->{$key} = $patch_desc[$field];
      }

      # Fill key from other fields
      else
      {
         $dbkey = $handle . $PATCH_FIELDS{$key};

         # MEM_INFO will be parsed by separate elements and the array from
         # these element will be returned
         if ($key eq 'MEM_INFO')
         {
            @meminfo = split(" ", $db{$dbkey});
            $res->{$key} = \@meminfo;
         }
         else
         {
            $res->{$key} = $db{$dbkey};
         }
      } ## end else [ if ($PATCH_FIELDS{$key...
   } ## end foreach my $key (@keys_tmp)
   $! = $OK;
   return $TRUE;
} ## end sub get_patch_info(@)

########
#Description: get one attribute for the patch
#INPUT:  $_[0] - reference to database hash
#        $_[1] - patch handle
#        $_[2] - name of the attribute
#OUTPUT: $! - error code
#        if $! = $OK the subroutine returns the value of the attribute
#           $! = 1 - there is no patch with such handle
#           $! = 2 - there is no such attribute
########
sub get_patch_attr(@)
{
   my %res;

   my $db     = $_[0];
   my $handle = $_[1];
   my $attr   = $_[2];

   # Create hash for request
   $res{$attr} = "";

   # Get information and check whether some errors happen
   if (get_patch_info($db, $handle, \%res) == $TRUE)
   {
      return $res{$attr};
   }
   else
   {
      return "";
   }
} ## end sub get_patch_attr(@)

#####################################################
## @fn get_fault_sp_str($command, $sp_name)
# @brief Generate output string for "FAULT" condition
# @param $sp_name - service pack name
# @param $command - command to generate output for.
# @return - "warning" string to put as output on "FAULT"
#           status of an operation

sub get_fault_sp_str($$)
{
   my ($command, $sp_name) = @_;
   return "$command command for SP $sp_name completed with some errors detected. Please refer to the $command output for more details.";
}

1;
__END__

=head1 NAME

patches_target - common data for target patch management tools.

=head1 SYNOPSIS

use patches_target;

=head1 LIMITATIONS

=head1 SEE ALSO

=cut
