###############################################################################
## @file   LogView.pm
# @brief   Log files inspection module.
# @details This module introduces common variables
#          and functions for log files overview
# @author  Ekaterina Bazhanova
# @date    February  2008
# @version 1.00 - Initial version


package recovery_pm::LogView;

use 5.008005;
use strict;
use warnings;
use common_variables;
use CommonInstallUtils;
use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.00';


## @var $dmesg 
#  Path to kernel messages file
my $dmesg     = $VARDIR . "/dmesg";

## @var $exit_mess
#  Message before return to the root menu
my $exit_mess = "Press <Enter> to continue:";

## @var $linuxbase
#  Path to the linuxbase subsystem log file
my $linuxbase = $VARDIR . "/nortel/linuxbase.log";

## @var $messages
#  Path to system specific messages file
my $messages  = $VARDIR . "/messages";

## @var $secure
#  Path to secure subsystem messages file 
my $secure    = $VARDIR . "/secure";

## @var $varDev
#  /var partition
my $varDev    = undef;
my $menu_str  = "Log files inspection";
my %actions = ('menu'     => {'string' => $menu_str,
                             'visible' => $TRUE,
                             'default' => $TRUE},
               'DefaultEntryPoint'     => {'sub'     => \&DefaultEntryPoint,
                             'visible' => $TRUE,
                             'default' => $FALSE},

              );
## @skipline
sub DefaultEntryPoint();
## @skipline
sub actions();

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
  return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub DefaultEntryPoint()
# @brief   Default enter point to log file inspection module.
# @details This function allows creating of the log file inspection menu.
# @param   None
# @return  None
# @author  Ekaterina Bazhanova
# @date    February  2008


sub DefaultEntryPoint()
{
   my @log_list = ("",$messages, $dmesg, $secure, $linuxbase);
   $varDev = defDev('/var/log');
   unless ($varDev)
   {
      print "Error: Cannot define device for /var/log partition\n";
      print $exit_mess;
      <STDIN>;
   }

   unless ( -d $RECDIR)
   {
      unless (mkdir($RECDIR))    { return $FALSE;}
   }   
   unless ( -d $VARDIR)
   {
      unless (mkdir($VARDIR))    { return $FALSE;}
   }
   unless (system($MOUNT, $varDev, $VARDIR) == 0)
   {
      return $FALSE;
   }

   while ($TRUE)
   {

      #Clear screen
      clear();
      print "$menu_str:\n";
      print <<HERE;
 You may choose for overview one of the following files: 
 1) System specific messages     (/var/log/messages)
 2) Kernel messages              (/var/log/dmesg)
 3) Secure subsystem messages    (/var/log/secure)
 4) Linuxbase subsystem log      (/var/log/nortel/linuxbase.log)
 5) Exit
HERE
      print "Select an option (1-5):";
      my $choice = <STDIN>;
      chomp($choice);
      if ($choice=~ /^\s*(\d+)\s*$/)
      {
        $choice  = $1;
      }
      else
      {
       print "Incorrect choice!\n";      
       next;
      }
      if (($choice > 0) && ($choice < 5))
      {
         clear();
         system "$NANO", "-rdonly", "$log_list[$choice]";
         print $exit_mess;
         <STDIN>;
      } ## end if ($choice == 1)     
      elsif ($choice == 5)
      {
         last;
      }
   } ## end while ($TRUE)
   system "$UMOUNT", "$VARDIR";
} ## end sub DefaultEntryPoint

1;
__END__
