###############################################################################
## @file   MemDiag.pm
# @brief   System memory diagnostic module.
# @details This module introduces functions which contains the instructions
#          for the system memory diagnostic
# @author  Ekaterina Bazhanova
# @date    February  2008
# @version 1.00 - Initial version


package recovery_pm::MemDiag;

use 5.008005;
use strict;
use warnings;
use common_variables;
use CommonInstallUtils;
use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.00';



## @var $exit_mess
#  Message before return to the root menu
my $exit_mess = "Press <Enter> to continue:";
## @var $menu_str
#  Name of the option for the recovery console main menu
my $menu_str   = "Memory Diagnostic";
my %actions = ('menu'     => {'string'     => $menu_str,
                             'visible' => $TRUE,
                             'default' => $TRUE},
               'DefaultEntryPoint'     => {'sub'     => \&DefaultEntryPoint,
                             'visible' => $TRUE,
                             'default' => $FALSE},

              );
## @skipline
sub MenuString();
## @skipline
sub DefaultEntryPoint();
## @skipline
sub memtest_msg();
## @skipline
sub actions();

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
  return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub DefaultEntryPoint()
# @brief   Default enter point to system memory diagnostic module.
# @details This function creates inststuction window of the memory
#          diagnostic module.
# @param   None
# @return  None
# @author  Ekaterina Bazhanova
# @date    February  2008

sub DefaultEntryPoint()
{

   clear();
   print "$menu_str:\n";
   memtest_msg();
   print $exit_mess;
   <STDIN>;

} ## end sub DefaultEntryPoint

###############################################################################
## @fn sub memtest_msg()
# @brief   Instruction messages.
# @details This function contains some instructions how to run Memtester 86+ .
# @param   None
# @return  None
# @author  Ekaterina Bazhanova
# @date    February  2008

sub memtest_msg()
{
   print <<HERE;

To perform system memory test do the following:
- Reboot system
- Type "memtest <Enter>" in the boot prompt field 
  
HERE
}  #end sub memtest_msg

1;
__END__
