###############################################################################
## @file   ShellCall.pm
# @brief   Recovery shell module.
# @details This module introduces common variables and functions
#          for recovery shell run
# @author  Ekaterina Bazhanova
# @date    February  2008
# @version 1.00 - Initial version

package recovery_pm::ShellCall;

use 5.008005;
use strict;
use warnings;
use common_variables;
use CommonInstallUtils;
use Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(&actions);
our $VERSION = '1.00';

## @var $menu_str
#  Name of the option for the recovery console main menu
my $menu_str = "Recovery shell";

my %actions = ('menu'     => {'string'     => $menu_str,
                              'visible'    => $TRUE,
                              'default'    => $TRUE},
               'DefaultEntryPoint'     => {'sub'     => \&DefaultEntryPoint,
                                           'visible' => $TRUE,
                                           'default' => $FALSE},

              );
## @skipline
sub DefaultEntryPoint();

## @skipline
sub actions();

###############################################################################
## @fn sub actions()
# @brief Hash of actions provided by module
# @details This method exported from the module returns the hash %actions
#          describing the actions which could be accessible for execution.
# @return %actions - the actions hash
sub actions()
{
  return %actions;
} ##end sub actions ()

###############################################################################
## @fn sub DefaultEntryPoint()
# @brief   Default enter point to recovery shell module.
# @details This function contains the recovery shell call.
# @param   None
# @return  None
# @author  Ekaterina Bazhanova
# @date    February  2008

sub DefaultEntryPoint()
{
   my $tty;
   #Clear screen
   clear();
   $tty = `$TTY`;
   chomp($tty);
   print "Welcome to Recovery Shell!\n(enter \"exit\" to exit)\n\n";
   system "PS1='recovery shell\# ' $SH 2>$tty";

} ## end sub DefaultEntryPoint

1;
__END__
